@@ -6,16 +6,17 @@ module.exports = {
   baseUrl: '/',
   onBrokenLinks: 'throw',
   onBrokenMarkdownLinks: 'warn',
-  favicon: 'https://atlasgo.io/uploads/favicon.ico',
   organizationName: 'ariga',
   projectName: 'atlas',
   themeConfig: {
-    gtag: {
-      trackingID: 'G-Z88N4TF03R'
-    },
     prism: {
       additionalLanguages: ['hcl'],
     },
+    algolia: {
+      appId: 'D158RRDJO1',
+      apiKey: "383b9efbb1486f812c8c023556b15396",
+      indexName: "atlasgo",
+    },
     navbar: {
       title: 'Atlas',
       logo: {
@@ -29,7 +30,7 @@ module.exports = {
           position: 'right',
         },
         {
-          href: 'https://github.com/ariga/atlas',
+          href: 'https://discord.gg/zZ6sWVg6NT',
           className: 'header-discord-link',
           position: 'right',
         },{
@@ -37,6 +38,21 @@ module.exports = {
           className: 'header-twitter-link',
           position: 'right',
         },
+        {
+          to: 'cli/getting-started/setting-up',
+          label: 'Docs',
+          position: 'left',
+        },
+        {
+          to: 'https://pkg.go.dev/ariga.io/atlas',
+          label: 'GoDoc',
+          position: 'left',
+        },
+        {
+          to: 'blog',
+          label: 'Blog',
+          position: 'left'
+        },
       ],
     },
     //
@@ -48,6 +64,7 @@ module.exports = {
             {"label": "Getting Started", "to": "cli/getting-started/setting-up"},
             {"label": "Data Definition Language ", "to": "ddl/intro"},
             {"label": "CLI Reference", "to": "cli/reference"},
+            {"label": "Blog", "to": "blog"},
             {"label": "About", "to": "about"},
           ]
         },
@@ -55,9 +72,17 @@ module.exports = {
           "title": "Community",
           "items": [
             {"label": "GitHub", "to": "https://github.com/ariga/atlas"},
-            {"label": "Discord", "to": "https://discord.com/invite/QhsmBAWzrC"},
+            {"label": "Discord", "to": "https://discord.gg/zZ6sWVg6NT"},
           ]
         },
+        {
+          "title": "Legal",
+          "items": [
+            {"label": "Privacy Policy", "to": "https://ariga.io/legal/privacy"},
+            {"label": "Terms of Service", "to": "https://ariga.io/legal/tos"},
+            {"label": "End User License", "to": "https://ariga.io/legal/atlas/eula"},
+          ]
+        }
       ],
       copyright: `
       Copyright © ${new Date().getFullYear()} The Atlas Authors.
@@ -68,7 +93,31 @@ module.exports = {
       <br/>
       `,
     },
+    announcementBar: {
+      id: 'announcementBar-1', // Increment on change
+      content: `️🚀 Sign up for a user testing session and receive exclusive Atlas swag, register <a target="_blank" rel="noopener noreferrer" href="https://calendly.com/ariga-user-testing/atlas-user-testing">here!</a>`,
+      isCloseable: true,
+    },
   },
+  plugins: [
+    [
+      '@docusaurus/plugin-client-redirects',
+      {
+        redirects: [
+          {
+            to: '/dev-database',
+            from: '/cli/dev-database',
+          },
+        ],
+      },
+    ],
+    [
+      require.resolve('docusaurus-gtm-plugin'),
+      {
+        id: 'GTM-T9GX8BR', // GTM Container ID
+      }
+    ],
+  ],
   presets: [
     [
       '@docusaurus/preset-classic',
@@ -80,6 +129,9 @@ module.exports = {
           showLastUpdateAuthor: false,
           showLastUpdateTime: false,
         },
+        gtag: {
+          trackingID: 'G-Z88N4TF03R'
+        },
         blog: {
           showReadingTime: true,
         },
