@@ -8,10 +8,12 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
+	"io"
+	"log"
 	"strings"
-	"sync"
 	"testing"
 
+	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/mysql"
 	"ariga.io/atlas/sql/schema"
 	"entgo.io/ent/dialect"
@@ -22,35 +24,73 @@ import (
 type myTest struct {
 	*testing.T
 	db      *sql.DB
-	drv     *mysql.Driver
+	drv     migrate.Driver
+	rrw     migrate.RevisionReadWriter
 	version string
 	port    int
 }
 
-var myTests struct {
-	sync.Once
+var myTests = struct {
 	drivers map[string]*myTest
+	ports   map[string]int
+}{
+	drivers: make(map[string]*myTest),
+	ports: map[string]int{
+		"mysql56":  3306,
+		"mysql57":  3307,
+		"mysql8":   3308,
+		"maria107": 4306,
+		"maria102": 4307,
+		"maria103": 4308,
+	},
+}
+
+func myInit(d string) []io.Closer {
+	var cs []io.Closer
+	if d != "" {
+		p, ok := myTests.ports[d]
+		if ok {
+			myTests.ports = map[string]int{d: p}
+		} else {
+			myTests.ports = make(map[string]int)
+		}
+	}
+	for version, port := range myTests.ports {
+		db, err := sql.Open("mysql", fmt.Sprintf("root:pass@tcp(localhost:%d)/test?parseTime=True", port))
+		if err != nil {
+			log.Fatalln(err)
+		}
+		cs = append(cs, db)
+		drv, err := mysql.Open(db)
+		if err != nil {
+			log.Fatalln(err)
+		}
+		myTests.drivers[version] = &myTest{db: db, drv: drv, version: version, port: port, rrw: &rrw{}}
+	}
+	return cs
 }
 
 func myRun(t *testing.T, fn func(*myTest)) {
-	myTests.Do(func() {
-		myTests.drivers = make(map[string]*myTest)
-		for version, port := range map[string]int{"56": 3306, "57": 3307, "8": 3308, "Maria107": 4306, "Maria102": 4307, "Maria103": 4308} {
-			db, err := sql.Open("mysql", fmt.Sprintf("root:pass@tcp(localhost:%d)/test?parseTime=True", port))
-			require.NoError(t, err)
-			drv, err := mysql.Open(db)
-			require.NoError(t, err)
-			myTests.drivers[version] = &myTest{db: db, drv: drv, version: version, port: port}
-		}
-	})
 	for version, tt := range myTests.drivers {
 		t.Run(version, func(t *testing.T) {
-			tt := &myTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port}
+			tt := &myTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port, rrw: tt.rrw}
 			fn(tt)
 		})
 	}
 }
 
+func TestMySQL_EntRevisions(t *testing.T) {
+	myRun(t, func(t *myTest) {
+		testEntRevisions(t)
+	})
+}
+
+func TestMySQL_Executor(t *testing.T) {
+	myRun(t, func(t *myTest) {
+		testExecutor(t)
+	})
+}
+
 func TestMySQL_AddDropTable(t *testing.T) {
 	myRun(t, func(t *myTest) {
 		testAddDrop(t)
@@ -353,6 +393,26 @@ func TestMySQL_ColumnBool(t *testing.T) {
 	})
 }
 
+func TestMySQL_ColumnCheck(t *testing.T) {
+	myRun(t, func(t *myTest) {
+		// Checks are not supported in all versions.
+		if t.version == "mysql56" || t.version == "mysql57" {
+			t.Skip()
+		}
+		usersT := &schema.Table{
+			Name:  "users",
+			Attrs: []schema.Attr{schema.NewCheck().SetName("users_c_check").SetExpr("c > 5")},
+			Columns: []*schema.Column{
+				{Name: "id", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "int"}}},
+				{Name: "c", Type: &schema.ColumnType{Type: &schema.IntegerType{T: "int"}}},
+			},
+		}
+		t.dropTables(usersT.Name)
+		t.migrate(&schema.AddTable{T: usersT})
+		ensureNoChange(t, usersT)
+	})
+}
+
 func TestMySQL_ForeignKey(t *testing.T) {
 	t.Run("ChangeAction", func(t *testing.T) {
 		myRun(t, func(t *myTest) {
@@ -459,6 +519,12 @@ func TestMySQL_Ent(t *testing.T) {
 	})
 }
 
+func TestMySQL_AdvisoryLock(t *testing.T) {
+	myRun(t, func(t *myTest) {
+		testAdvisoryLock(t.T, t.drv.(schema.Locker))
+	})
+}
+
 func TestMySQL_HCL(t *testing.T) {
 	full := `
 schema "test" {
@@ -521,21 +587,107 @@ func TestMySQL_CLI(t *testing.T) {
 		myRun(t, func(t *myTest) {
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLISchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn(), mysql.UnmarshalHCL)
+			testCLISchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"), mysql.UnmarshalHCL)
 		})
 	})
 	t.Run("SchemaApply", func(t *testing.T) {
 		myRun(t, func(t *myTest) {
 			attrs := t.defaultAttrs()
 			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
-			testCLISchemaApply(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn())
+			testCLISchemaApply(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"))
+		})
+	})
+	t.Run("SchemaApplyWithVars", func(t *testing.T) {
+		h := `
+variable "tenant" {
+	type = string
+}
+schema "tenant" {
+	name = var.tenant
+}
+table "users" {
+	schema = schema.tenant
+	column "id" {
+		type = int
+	}
+}
+`
+		myRun(t, func(t *myTest) {
+			testCLISchemaApply(t, h, t.dsn("test"), "--var", "tenant=test")
+		})
+	})
+	t.Run("SchemaApplyDryRun", func(t *testing.T) {
+		myRun(t, func(t *myTest) {
+			attrs := t.defaultAttrs()
+			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
+			testCLISchemaApplyDry(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"))
+		})
+	})
+	t.Run("SchemaDiffRun", func(t *testing.T) {
+		myRun(t, func(t *myTest) {
+			testCLISchemaDiff(t, t.dsn("test"))
+		})
+	})
+	t.Run("SchemaApplyAutoApprove", func(t *testing.T) {
+		myRun(t, func(t *myTest) {
+			attrs := t.defaultAttrs()
+			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
+			testCLISchemaApplyAutoApprove(t, fmt.Sprintf(h, charset.V, collate.V), t.dsn("test"))
+		})
+	})
+}
+
+func TestMySQL_CLI_MultiSchema(t *testing.T) {
+	h := `
+			schema "test" {
+				charset   = "%s"
+				collation = "%s"
+			}
+			table "users" {
+				schema = schema.test
+				column "id" {
+					type = int
+				}
+				primary_key {
+					columns = [table.users.column.id]
+				}
+			}
+			schema "test2" {
+				charset   = "%s"
+				collation = "%s"
+			}
+			table "users" {
+				schema = schema.test2
+				column "id" {
+					type = int
+				}
+				primary_key {
+					columns = [table.users.column.id]
+				}
+			}`
+	t.Run("SchemaInspect", func(t *testing.T) {
+		myRun(t, func(t *myTest) {
+			t.dropSchemas("test2")
+			t.dropTables("users")
+			attrs := t.defaultAttrs()
+			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
+			testCLIMultiSchemaInspect(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.UnmarshalHCL)
+		})
+	})
+	t.Run("SchemaApply", func(t *testing.T) {
+		myRun(t, func(t *myTest) {
+			t.dropSchemas("test2")
+			t.dropTables("users")
+			attrs := t.defaultAttrs()
+			charset, collate := attrs[0].(*schema.Charset), attrs[1].(*schema.Collation)
+			testCLIMultiSchemaApply(t, fmt.Sprintf(h, charset.V, collate.V, charset.V, collate.V), t.dsn(""), []string{"test", "test2"}, mysql.UnmarshalHCL)
 		})
 	})
 }
 
 func TestMySQL_HCL_Realm(t *testing.T) {
 	myRun(t, func(t *myTest) {
-		t.dropDB("second")
+		t.dropSchemas("second")
 		realm := t.loadRealm()
 		hcl, err := mysql.MarshalHCL(realm)
 		require.NoError(t, err)
@@ -587,44 +739,47 @@ func TestMySQL_Sanity(t *testing.T) {
 		ddl := `
 create table atlas_types_sanity
 (
-    tBit                 bit(10)              default b'100'            null,
-    tInt                 int(10)              default 4             not null,
-    tTinyInt             tinyint(10)          default 8                 null,
-    tSmallInt            smallint(10)         default 2                 null,
-    tMediumInt           mediumint(10)        default 11                null,
-    tBigInt              bigint(10)           default 4                 null,
-    tDecimal             decimal              default 4                 null,
-    tNumeric             numeric              default 4             not null,
-    tFloat               float(10, 0)         default 4                 null,
-    tDouble              double(10, 0)        default 4                 null,
-    tReal                double(10, 0)        default 4                 null,
-    tTimestamp           timestamp            default CURRENT_TIMESTAMP null,
-    tDate                date                                           null,
-    tTime                time                                           null,
-    tDateTime            datetime                                       null,
-    tYear                year                                           null,
-    tVarchar             varchar(10)          default 'Titan'           null,
-    tChar                char(25)             default 'Olimpia'     not null,
-    tVarBinary           varbinary(30)        default 'Titan'           null,
-    tBinary              binary(5)            default 'Titan'           null,
-    tBlob                blob(5)              default                   null,
-    tTinyBlob            tinyblob                                       null,
-    tMediumBlob          mediumblob           default                   null,
-    tLongBlob            longblob             default                   null,
-    tText                text(13)             default                   null,
-    tTinyText            tinytext             default                   null,
-    tMediumText          mediumtext           default                   null,
-    tLongText            longtext             default                   null,
-    tEnum                enum('a','b')        default                   null,
-    tSet                 set('a','b')         default                   null,
-    tGeometry            geometry             default                   null,
-    tPoint               point                default                   null,
-    tMultiPoint          multipoint           default                   null,
-    tLineString          linestring           default                   null,
-    tMultiLineString     multilinestring      default                   null,
-    tPolygon             polygon              default                   null,
-    tMultiPolygon        multipolygon         default                   null,
-    tGeometryCollection  geometrycollection   default                   null
+    tBit                        bit(10)              default b'100'                                              null,
+    tInt                        int(10)              default 4                                               not null,
+    tTinyInt                    tinyint(10)          default 8                                                   null,
+    tSmallInt                   smallint(10)         default 2                                                   null,
+    tMediumInt                  mediumint(10)        default 11                                                  null,
+    tBigInt                     bigint(10)           default 4                                                   null,
+    tDecimal                    decimal              default 4                                                   null,
+    tNumeric                    numeric              default 4                                               not null,
+    tFloat                      float(10, 0)         default 4                                                   null,
+    tDouble                     double(10, 0)        default 4                                                   null,
+    tReal                       double(10, 0)        default 4                                                   null,
+    tTimestamp                  timestamp            default CURRENT_TIMESTAMP                                   null,
+    tTimestampFraction          timestamp(6)         default CURRENT_TIMESTAMP(6)                                null,
+    tTimestampOnUpdate          timestamp            default CURRENT_TIMESTAMP    ON UPDATE CURRENT_TIMESTAMP    null,
+    tTimestampFractionOnUpdate  timestamp(6)         default CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6) null,
+    tDate                       date                                                                             null,
+    tTime                       time                                                                             null,
+    tDateTime                   datetime                                                                         null,
+    tYear                       year                                                                             null,
+    tVarchar                    varchar(10)          default 'Titan'                                             null,
+    tChar                       char(25)             default 'Olimpia'                                       not null,
+    tVarBinary                  varbinary(30)        default 'Titan'                                             null,
+    tBinary                     binary(5)            default 'Titan'                                             null,
+    tBlob                       blob(5)              default                                                     null,
+    tTinyBlob                   tinyblob                                                                         null,
+    tMediumBlob                 mediumblob           default                                                     null,
+    tLongBlob                   longblob             default                                                     null,
+    tText                       text(13)             default                                                     null,
+    tTinyText                   tinytext             default                                                     null,
+    tMediumText                 mediumtext           default                                                     null,
+    tLongText                   longtext             default                                                     null,
+    tEnum                       enum('a','b')        default                                                     null,
+    tSet                        set('a','b')         default                                                     null,
+    tGeometry                   geometry             default                                                     null,
+    tPoint                      point                default                                                     null,
+    tMultiPoint                 multipoint           default                                                     null,
+    tLineString                 linestring           default                                                     null,
+    tMultiLineString            multilinestring      default                                                     null,
+    tPolygon                    polygon              default                                                     null,
+    tMultiPolygon               multipolygon         default                                                     null,
+    tGeometryCollection         geometrycollection   default                                                     null
 ) CHARSET = latin1 COLLATE latin1_swedish_ci;
 `
 		myRun(t, func(t *myTest) {
@@ -651,31 +806,31 @@ create table atlas_types_sanity
 					{
 						Name: "tInt",
 						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "int", Unsigned: false},
-							Raw: t.valueByVersion(map[string]string{"8": "int"}, "int(10)"), Null: false},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "int"}, "int(10)"), Null: false},
 						Default: &schema.Literal{V: "4"},
 					},
 					{
 						Name: "tTinyInt",
 						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "tinyint", Unsigned: false},
-							Raw: t.valueByVersion(map[string]string{"8": "tinyint"}, "tinyint(10)"), Null: true},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "tinyint"}, "tinyint(10)"), Null: true},
 						Default: &schema.Literal{V: "8"},
 					},
 					{
 						Name: "tSmallInt",
 						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "smallint", Unsigned: false},
-							Raw: t.valueByVersion(map[string]string{"8": "smallint"}, "smallint(10)"), Null: true},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "smallint"}, "smallint(10)"), Null: true},
 						Default: &schema.Literal{V: "2"},
 					},
 					{
 						Name: "tMediumInt",
 						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "mediumint", Unsigned: false},
-							Raw: t.valueByVersion(map[string]string{"8": "mediumint"}, "mediumint(10)"), Null: true},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "mediumint"}, "mediumint(10)"), Null: true},
 						Default: &schema.Literal{V: "11"},
 					},
 					{
 						Name: "tBigInt",
 						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "bigint", Unsigned: false},
-							Raw: t.valueByVersion(map[string]string{"8": "bigint"}, "bigint(10)"), Null: true},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "bigint"}, "bigint(10)"), Null: true},
 						Default: &schema.Literal{V: "4"},
 					},
 					{
@@ -715,12 +870,70 @@ create table atlas_types_sanity
 						Default: &schema.RawExpr{
 							X: func() string {
 								if t.mariadb() {
-									return "current_timestamp()"
+									return "(current_timestamp())"
 								}
 								return "CURRENT_TIMESTAMP"
 							}(),
 						},
 					},
+					{
+						Name: "tTimestampFraction",
+						Type: &schema.ColumnType{Type: &schema.TimeType{T: "timestamp", Precision: intp(6)},
+							Raw: "timestamp(6)", Null: true},
+						Default: &schema.RawExpr{
+							X: func() string {
+								if t.mariadb() {
+									return "(current_timestamp(6))"
+								}
+								return "CURRENT_TIMESTAMP(6)"
+							}(),
+						},
+					},
+					{
+						Name: "tTimestampOnUpdate",
+						Type: &schema.ColumnType{Type: &schema.TimeType{T: "timestamp"},
+							Raw: "timestamp", Null: true},
+						Default: &schema.RawExpr{
+							X: func() string {
+								if t.mariadb() {
+									return "(current_timestamp())"
+								}
+								return "CURRENT_TIMESTAMP"
+							}(),
+						},
+						Attrs: []schema.Attr{
+							&mysql.OnUpdate{
+								A: func() string {
+									if t.mariadb() {
+										return "current_timestamp()"
+									}
+									return "CURRENT_TIMESTAMP"
+								}(),
+							},
+						},
+					},
+					{
+						Name: "tTimestampFractionOnUpdate",
+						Type: &schema.ColumnType{Type: &schema.TimeType{T: "timestamp", Precision: intp(6)}, Raw: "timestamp(6)", Null: true},
+						Default: &schema.RawExpr{
+							X: func() string {
+								if t.mariadb() {
+									return "(current_timestamp(6))"
+								}
+								return "CURRENT_TIMESTAMP(6)"
+							}(),
+						},
+						Attrs: []schema.Attr{
+							&mysql.OnUpdate{
+								A: func() string {
+									if t.mariadb() {
+										return "current_timestamp(6)"
+									}
+									return "CURRENT_TIMESTAMP(6)"
+								}(),
+							},
+						},
+					},
 					{
 						Name: "tDate",
 						Type: &schema.ColumnType{Type: &schema.TimeType{T: "date"},
@@ -738,8 +951,19 @@ create table atlas_types_sanity
 					},
 					{
 						Name: "tYear",
-						Type: &schema.ColumnType{Type: &schema.TimeType{T: "year"},
-							Raw: t.valueByVersion(map[string]string{"8": "year"}, "year(4)"), Null: true},
+						Type: &schema.ColumnType{
+							Type: &schema.TimeType{
+								T: "year",
+								Precision: func() *int {
+									// From MySQL 8.0.19, display width is deprecated in YEAR types.
+									if t.version == "mysql8" {
+										return nil
+									}
+									p := 4
+									return &p
+								}(),
+							},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "year"}, "year(4)"), Null: true},
 					},
 					{
 						Name: "tVarchar",
@@ -765,13 +989,13 @@ create table atlas_types_sanity
 						Name: "tVarBinary",
 						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "varbinary", Size: 30},
 							Raw: "varbinary(30)", Null: true},
-						Default: &schema.Literal{V: t.valueByVersion(map[string]string{"8": "0x546974616E"}, t.quoted("Titan"))},
+						Default: &schema.Literal{V: t.valueByVersion(map[string]string{"mysql8": "0x546974616E"}, t.quoted("Titan"))},
 					},
 					{
 						Name: "tBinary",
 						Type: &schema.ColumnType{Type: &schema.BinaryType{T: "binary", Size: 5},
 							Raw: "binary(5)", Null: true},
-						Default: &schema.Literal{V: t.valueByVersion(map[string]string{"8": "0x546974616E"}, t.quoted("Titan"))},
+						Default: &schema.Literal{V: t.valueByVersion(map[string]string{"mysql8": "0x546974616E"}, t.quoted("Titan"))},
 					},
 					{
 						Name: "tBlob",
@@ -885,8 +1109,8 @@ create table atlas_types_sanity
 					{
 						Name: "tGeometryCollection",
 						Type: &schema.ColumnType{Type: &schema.SpatialType{T: t.valueByVersion(
-							map[string]string{"8": "geomcollection"}, "geometrycollection")},
-							Raw: t.valueByVersion(map[string]string{"8": "geomcollection"},
+							map[string]string{"mysql8": "geomcollection"}, "geometrycollection")},
+							Raw: t.valueByVersion(map[string]string{"mysql8": "geomcollection"},
 								"geometrycollection"), Null: true},
 					},
 				},
@@ -904,7 +1128,7 @@ create table atlas_types_sanity
 ) CHARSET = latin1 COLLATE latin1_swedish_ci;
 `
 		myRun(t, func(t *myTest) {
-			if t.version == "56" {
+			if t.version == "mysql56" {
 				return
 			}
 			t.dropTables(n)
@@ -917,11 +1141,11 @@ create table atlas_types_sanity
 			expected := schema.Table{
 				Name: n,
 				Attrs: func() []schema.Attr {
-					if t.version == "Maria107" {
+					if t.version == "maria107" {
 						return []schema.Attr{
 							&schema.Charset{V: "latin1"},
 							&schema.Collation{V: "latin1_swedish_ci"},
-							&schema.Check{Name: "tJSON", Expr: "json_valid(`tJSON`)", Attrs: []schema.Attr{&mysql.Enforced{}}},
+							&schema.Check{Name: "tJSON", Expr: "json_valid(`tJSON`)"},
 						}
 					}
 					return []schema.Attr{
@@ -934,9 +1158,9 @@ create table atlas_types_sanity
 					func() *schema.Column {
 						c := &schema.Column{Name: "tJSON", Type: &schema.ColumnType{Type: &schema.JSONType{T: "json"}, Raw: "json", Null: true}}
 						switch t.version {
-						case "Maria107":
+						case "maria107":
 							c.Attrs = []schema.Attr{}
-						case "Maria102", "Maria103":
+						case "maria102", "maria103":
 							c.Type.Raw = "longtext"
 							c.Type.Type = &schema.StringType{T: "longtext"}
 							c.Attrs = []schema.Attr{
@@ -960,12 +1184,28 @@ create table atlas_types_sanity
 	})
 }
 
-func (t *myTest) dsn() string {
+func (t *myTest) url() string {
+	return t.dsn("")
+}
+
+func (t *myTest) dsn(dbname string) string {
 	d := "mysql"
+	pass := ":pass"
+	if t.tidb() {
+		pass = ""
+	}
 	if t.mariadb() {
 		d = "mariadb"
 	}
-	return fmt.Sprintf("%s://root:pass@tcp(localhost:%d)/test", d, t.port)
+	return fmt.Sprintf("%s://root%s@localhost:%d/%s", d, pass, t.port, dbname)
+}
+
+func (t *myTest) driver() migrate.Driver {
+	return t.drv
+}
+
+func (t *myTest) revisionsStorage() migrate.RevisionReadWriter {
+	return t.rrw
 }
 
 func (t *myTest) applyHcl(spec string) {
@@ -974,6 +1214,7 @@ func (t *myTest) applyHcl(spec string) {
 	err := mysql.UnmarshalHCL([]byte(spec), &desired)
 	require.NoError(t, err)
 	existing := realm.Schemas[0]
+	require.NoError(t, err)
 	diff, err := t.drv.SchemaDiff(existing, &desired)
 	require.NoError(t, err)
 	err = t.drv.ApplyChanges(context.Background(), diff)
@@ -1009,10 +1250,12 @@ func (t *myTest) dropTables(names ...string) {
 	})
 }
 
-func (t *myTest) dropDB(names ...string) {
+func (t *myTest) dropSchemas(names ...string) {
 	t.Cleanup(func() {
-		_, err := t.db.Exec("DROP DATABASE IF EXISTS " + strings.Join(names, ", "))
-		require.NoError(t.T, err, "drop db %q", names)
+		for _, n := range names {
+			_, err := t.db.Exec("DROP DATABASE IF EXISTS " + n)
+			require.NoError(t.T, err, "drop db %q", names)
+		}
 	})
 }
 
@@ -1093,6 +1336,34 @@ func (t *myTest) posts() *schema.Table {
 	return postsT
 }
 
+func (t *myTest) revisions() *schema.Table {
+	versionsT := &schema.Table{
+		Name: "atlas_schema_revisions",
+		Columns: []*schema.Column{
+			{Name: "version", Type: &schema.ColumnType{Type: &schema.StringType{T: t.valueByVersion(map[string]string{"mysql56": "varchar(191)"}, "varchar(255)")}}},
+			{Name: "description", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar(255)"}}},
+			{Name: "execution_state", Type: &schema.ColumnType{Type: &schema.EnumType{Values: []string{"ongoing", "ok", "error"}}}},
+			{Name: "executed_at", Type: &schema.ColumnType{Type: &schema.TimeType{T: "timestamp"}, Raw: "timestamp", Null: t.version != "mysql8"}},
+			{Name: "execution_time", Type: &schema.ColumnType{Type: &schema.IntegerType{T: t.valueByVersion(map[string]string{"mysql56": "bigint(20)"}, "bigint")}}},
+			{Name: "hash", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar(255)"}}},
+			{Name: "operator_version", Type: &schema.ColumnType{Type: &schema.StringType{T: "varchar(255)"}}},
+		},
+		Attrs: []schema.Attr{
+			&schema.Charset{V: "utf8mb4"},       // because of Ent
+			&schema.Collation{V: "utf8mb4_bin"}, // because of Ent
+		},
+	}
+	versionsT.PrimaryKey = &schema.Index{Parts: []*schema.IndexPart{{C: versionsT.Columns[0]}}}
+	var metaType schema.ColumnType
+	if t.version == "mysql56" {
+		metaType = schema.ColumnType{Type: &schema.BinaryType{T: "longblob"}}
+	} else {
+		metaType = schema.ColumnType{Type: &schema.JSONType{T: "json"}}
+	}
+	versionsT.Columns = append(versionsT.Columns, &schema.Column{Name: "meta", Type: &metaType})
+	return versionsT
+}
+
 func (t *myTest) valueByVersion(values map[string]string, defaults string) string {
 	if v, ok := values[t.version]; ok {
 		return v
@@ -1100,6 +1371,13 @@ func (t *myTest) valueByVersion(values map[string]string, defaults string) strin
 	return defaults
 }
 
+func (t *myTest) intByVersion(values map[string]int, defaults int) int {
+	if v, ok := values[t.version]; ok {
+		return v
+	}
+	return defaults
+}
+
 func (t *myTest) quoted(s string) string {
 	c := "\""
 	if t.mariadb() {
@@ -1132,7 +1410,8 @@ func (t *myTest) loadTable(name string) *schema.Table {
 	return table
 }
 
-func (t *myTest) mariadb() bool { return strings.HasPrefix(t.version, "Maria") }
+func (t *myTest) mariadb() bool { return strings.HasPrefix(t.version, "maria") }
+func (t *myTest) tidb() bool    { return strings.HasPrefix(t.version, "tidb") }
 
 // defaultConfig returns the default charset and
 // collation configuration based on the MySQL version.
@@ -1142,10 +1421,13 @@ func (t *myTest) defaultAttrs() []schema.Attr {
 		collation = "latin1_swedish_ci"
 	)
 	switch {
-	case t.version == "8":
+	case strings.Contains(t.version, "tidb"):
+		charset = "utf8mb4"
+		collation = "utf8mb4_bin"
+	case t.version == "mysql8":
 		charset = "utf8mb4"
 		collation = "utf8mb4_0900_ai_ci"
-	case t.version == "Maria107":
+	case t.version == "maria107":
 		charset = "utf8mb4"
 		collation = "utf8mb4_general_ci"
 	}
@@ -1180,3 +1462,5 @@ func rmCreateStmt(t *schema.Table) {
 		}
 	}
 }
+
+func intp(i int) *int { return &i }
