@@ -33,50 +33,53 @@ type (
 	CheckSpecFunc         func(*schema.Check) *sqlspec.Check
 )
 
-// Realm converts the schemas and tables into a schema.Realm.
-func Realm(schemas []*sqlspec.Schema, tables []*sqlspec.Table, convertTable ConvertTableFunc) (*schema.Realm, error) {
-	r := &schema.Realm{}
+// Scan populates the Realm from the schemas and table specs.
+func Scan(r *schema.Realm, schemas []*sqlspec.Schema, tables []*sqlspec.Table, convertTable ConvertTableFunc) error {
+	// Build the schemas.
 	for _, schemaSpec := range schemas {
-		var schemaTables []*sqlspec.Table
+		sch := &schema.Schema{Name: schemaSpec.Name, Realm: r}
 		for _, tableSpec := range tables {
 			name, err := SchemaName(tableSpec.Schema)
 			if err != nil {
-				return nil, fmt.Errorf("specutil: cannot extract schema name for table %q: %w", tableSpec.Name, err)
+				return fmt.Errorf("specutil: cannot extract schema name for table %q: %w", tableSpec.Name, err)
 			}
 			if name == schemaSpec.Name {
-				schemaTables = append(schemaTables, tableSpec)
+				tbl, err := convertTable(tableSpec, sch)
+				if err != nil {
+					return err
+				}
+				sch.Tables = append(sch.Tables, tbl)
 			}
 		}
-		sch, err := Schema(schemaSpec, schemaTables, convertTable)
-		if err != nil {
-			return nil, err
-		}
 		r.Schemas = append(r.Schemas, sch)
 	}
-	return r, nil
+	// Link the foreign keys.
+	for _, sch := range r.Schemas {
+		for _, tbl := range sch.Tables {
+			tableSpec, err := findTableSpec(tables, sch.Name, tbl.Name)
+			if err != nil {
+				return err
+			}
+			if err := linkForeignKeys(tbl, sch, tableSpec); err != nil {
+				return err
+			}
+		}
+	}
+	return nil
 }
 
-// Schema converts a sqlspec.Schema with its relevant []sqlspec.Tables
-// into a schema.Schema.
-func Schema(spec *sqlspec.Schema, tables []*sqlspec.Table, convertTable ConvertTableFunc) (*schema.Schema, error) {
-	sch := &schema.Schema{
-		Name: spec.Name,
-	}
-	m := make(map[*schema.Table]*sqlspec.Table)
-	for _, ts := range tables {
-		table, err := convertTable(ts, sch)
+// findTableSpec searches tableSpecs for a spec of a table named tableName in a schema named schemaName.
+func findTableSpec(tableSpecs []*sqlspec.Table, schemaName, tableName string) (*sqlspec.Table, error) {
+	for _, tbl := range tableSpecs {
+		n, err := SchemaName(tbl.Schema)
 		if err != nil {
 			return nil, err
 		}
-		sch.Tables = append(sch.Tables, table)
-		m[table] = ts
-	}
-	for _, tbl := range sch.Tables {
-		if err := LinkForeignKeys(tbl, sch, m[tbl]); err != nil {
-			return nil, err
+		if n == schemaName && tbl.Name == tableName {
+			return tbl, nil
 		}
 	}
-	return sch, nil
+	return nil, fmt.Errorf("table %s.%s not found", schemaName, tableName)
 }
 
 // Table converts a sqlspec.Table to a schema.Table. Table conversion is done without converting
@@ -151,22 +154,50 @@ func Column(spec *sqlspec.Column, conv ConvertTypeFunc) (*schema.Column, error)
 	return out, err
 }
 
-// Index converts a sqlspec.Index to a schema.Index.
-func Index(spec *sqlspec.Index, parent *schema.Table) (*schema.Index, error) {
-	parts := make([]*schema.IndexPart, 0, len(spec.Columns))
-	for seqno, c := range spec.Columns {
-		cn, err := columnName(c)
-		if err != nil {
-			return nil, fmt.Errorf("specutil: failed converting column to index: %w", err)
+// Index converts a sqlspec.Index to a schema.Index. The optional arguments allow
+// passing functions for mutating the created index-part (e.g. add attributes).
+func Index(spec *sqlspec.Index, parent *schema.Table, partFns ...func(*sqlspec.IndexPart, *schema.IndexPart) error) (*schema.Index, error) {
+	parts := make([]*schema.IndexPart, 0, len(spec.Columns)+len(spec.Parts))
+	switch n, m := len(spec.Columns), len(spec.Parts); {
+	case n == 0 && m == 0:
+		return nil, fmt.Errorf("missing definition for index %q", spec.Name)
+	case n > 0 && m > 0:
+		return nil, fmt.Errorf(`multiple definitions for index %q, use "columns" or "on"`, spec.Name)
+	case n > 0:
+		for i, c := range spec.Columns {
+			c, err := ColumnByRef(parent, c)
+			if err != nil {
+				return nil, err
+			}
+			parts = append(parts, &schema.IndexPart{
+				SeqNo: i,
+				C:     c,
+			})
 		}
-		col, ok := parent.Column(cn)
-		if !ok {
-			return nil, fmt.Errorf("specutil: unknown column %q in table %q", cn, parent.Name)
+	case m > 0:
+		for i, p := range spec.Parts {
+			part := &schema.IndexPart{SeqNo: i, Desc: p.Desc}
+			switch {
+			case p.Column == nil && p.Expr == "":
+				return nil, fmt.Errorf(`"column" or "expr" are required for index %q at position %d`, spec.Name, i)
+			case p.Column != nil && p.Expr != "":
+				return nil, fmt.Errorf(`cannot use both "column" and "expr" in index %q at position %d`, spec.Name, i)
+			case p.Expr != "":
+				part.X = &schema.RawExpr{X: p.Expr}
+			case p.Column != nil:
+				c, err := ColumnByRef(parent, p.Column)
+				if err != nil {
+					return nil, err
+				}
+				part.C = c
+			}
+			for _, f := range partFns {
+				if err := f(p, part); err != nil {
+					return nil, err
+				}
+			}
+			parts = append(parts, part)
 		}
-		parts = append(parts, &schema.IndexPart{
-			SeqNo: seqno,
-			C:     col,
-		})
 	}
 	i := &schema.Index{
 		Name:   spec.Name,
@@ -192,17 +223,13 @@ func Check(spec *sqlspec.Check) (*schema.Check, error) {
 func PrimaryKey(spec *sqlspec.PrimaryKey, parent *schema.Table) (*schema.Index, error) {
 	parts := make([]*schema.IndexPart, 0, len(spec.Columns))
 	for seqno, c := range spec.Columns {
-		n, err := columnName(c)
+		c, err := ColumnByRef(parent, c)
 		if err != nil {
-			return nil, fmt.Errorf("sqlspec: cannot get column name %q as primary key for table %q", c.V, parent.Name)
-		}
-		pkc, ok := parent.Column(n)
-		if !ok {
-			return nil, fmt.Errorf("sqlspec: cannot set column %q as primary key for table %q", n, parent.Name)
+			return nil, nil
 		}
 		parts = append(parts, &schema.IndexPart{
 			SeqNo: seqno,
-			C:     pkc,
+			C:     c,
 		})
 	}
 	return &schema.Index{
@@ -211,85 +238,48 @@ func PrimaryKey(spec *sqlspec.PrimaryKey, parent *schema.Table) (*schema.Index,
 	}, nil
 }
 
-// LinkForeignKeys creates the foreign keys defined in the Table's spec by creating references
-// to column in the provided Schema. It is assumed that the schema contains all of the tables
-// referenced by the FK definitions in the spec.
-func LinkForeignKeys(tbl *schema.Table, sch *schema.Schema, table *sqlspec.Table) error {
+// linkForeignKeys creates the foreign keys defined in the Table's spec by creating references
+// to column in the provided Schema. It is assumed that all tables referenced FK definitions in the spec
+// are reachable from the provided schema or its connected realm.
+func linkForeignKeys(tbl *schema.Table, sch *schema.Schema, table *sqlspec.Table) error {
 	for _, spec := range table.ForeignKeys {
-		fk := &schema.ForeignKey{
-			Symbol:   spec.Symbol,
-			Table:    tbl,
-			OnUpdate: spec.OnUpdate,
-			OnDelete: spec.OnDelete,
+		fk := &schema.ForeignKey{Symbol: spec.Symbol, Table: tbl}
+		if spec.OnUpdate != nil {
+			fk.OnUpdate = schema.ReferenceOption(FromVar(spec.OnUpdate.V))
+		}
+		if spec.OnDelete != nil {
+			fk.OnDelete = schema.ReferenceOption(FromVar(spec.OnDelete.V))
+		}
+		if n, m := len(spec.Columns), len(spec.RefColumns); n != m {
+			return fmt.Errorf("sqlspec: number of referencing and referenced columns do not match for foreign-key %q", fk.Symbol)
 		}
 		for _, ref := range spec.Columns {
-			col, err := resolveCol(ref, sch)
+			c, err := ColumnByRef(tbl, ref)
 			if err != nil {
 				return err
 			}
-			fk.Columns = append(fk.Columns, col)
+			fk.Columns = append(fk.Columns, c)
 		}
-		if len(spec.RefColumns) == 0 {
-			return fmt.Errorf("sqlspec: missing reference (parent) columns for foreign key: %q", spec.Symbol)
-		}
-		name, err := tableName(spec.RefColumns[0])
-		if err != nil {
-			return err
-		}
-		t, ok := sch.Table(name)
-		if !ok {
-			return fmt.Errorf("sqlspec: undefined table %q for foreign key: %q", name, spec.Symbol)
-		}
-		fk.RefTable = t
-		for _, ref := range spec.RefColumns {
-			col, err := resolveCol(ref, sch)
+		for i, ref := range spec.RefColumns {
+			t, c, err := externalRef(ref, sch)
+			if isLocalRef(ref) {
+				t = fk.Table
+				c, err = ColumnByRef(fk.Table, ref)
+			}
 			if err != nil {
 				return err
 			}
-			fk.RefColumns = append(fk.RefColumns, col)
+			if i > 0 && fk.RefTable != t {
+				return fmt.Errorf("sqlspec: more than 1 table was referenced for foreign-key %q", fk.Symbol)
+			}
+			fk.RefTable = t
+			fk.RefColumns = append(fk.RefColumns, c)
 		}
 		tbl.ForeignKeys = append(tbl.ForeignKeys, fk)
 	}
 	return nil
 }
 
-func resolveCol(ref *schemaspec.Ref, sch *schema.Schema) (*schema.Column, error) {
-	t, err := tableName(ref)
-	if err != nil {
-		return nil, fmt.Errorf("sqlspec: table %q not found", ref.V)
-	}
-	tbl, ok := sch.Table(t)
-	if !ok {
-		return nil, fmt.Errorf("sqlspec: table %q not found", t)
-	}
-	c, err := columnName(ref)
-	if err != nil {
-		return nil, fmt.Errorf("sqlspec: column %q not found", ref.V)
-	}
-	col, ok := tbl.Column(c)
-	if !ok {
-		return nil, fmt.Errorf("sqlspec: column %q not found in table %q", c, t)
-	}
-	return col, nil
-}
-
-// FromRealm converts a schema.Realm into []sqlspec.Schema and []sqlspec.Table.
-func FromRealm(r *schema.Realm, fn TableSpecFunc) ([]*sqlspec.Schema, []*sqlspec.Table, error) {
-	var (
-		schemas []*sqlspec.Schema
-		tables  []*sqlspec.Table
-	)
-	for _, sch := range r.Schemas {
-		schemaSpec, tableSpecs, err := FromSchema(sch, fn)
-		if err != nil {
-			return nil, nil, fmt.Errorf("specutil: cannot convert schema %q: %w", sch.Name, err)
-		}
-		schemas = append(schemas, schemaSpec)
-		tables = append(tables, tableSpecs...)
-	}
-	return schemas, tables, nil
-}
-
 // FromSchema converts a schema.Schema into sqlspec.Schema and []sqlspec.Table.
 func FromSchema(s *schema.Schema, fn TableSpecFunc) (*sqlspec.Schema, []*sqlspec.Table, error) {
 	spec := &sqlspec.Schema{
@@ -356,7 +346,7 @@ func FromTable(t *schema.Table, colFn ColumnSpecFunc, pkFn PrimaryKeySpecFunc, i
 func FromPrimaryKey(s *schema.Index) (*sqlspec.PrimaryKey, error) {
 	c := make([]*schemaspec.Ref, 0, len(s.Parts))
 	for _, v := range s.Parts {
-		c = append(c, colRef(v.C.Name, s.Table.Name))
+		c = append(c, ColumnRef(v.C.Name))
 	}
 	return &sqlspec.PrimaryKey{
 		Columns: c,
@@ -388,6 +378,49 @@ func FromColumn(col *schema.Column, columnTypeSpec ColumnTypeSpecFunc) (*sqlspec
 	return spec, nil
 }
 
+// FromGenExpr returns the spec for a generated expression.
+func FromGenExpr(x schema.GeneratedExpr, t func(string) string) *schemaspec.Resource {
+	return &schemaspec.Resource{
+		Type: "as",
+		Attrs: []*schemaspec.Attr{
+			StrAttr("expr", x.Expr),
+			VarAttr("type", t(x.Type)),
+		},
+	}
+}
+
+// ConvertGenExpr converts the "as" attribute or the block under the given resource.
+func ConvertGenExpr(r *schemaspec.Resource, c *schema.Column, t func(string) string) error {
+	asA, okA := r.Attr("as")
+	asR, okR := r.Resource("as")
+	switch {
+	case okA && okR:
+		return fmt.Errorf("multiple as definitions for column %q", c.Name)
+	case okA:
+		expr, err := asA.String()
+		if err != nil {
+			return err
+		}
+		c.Attrs = append(c.Attrs, &schema.GeneratedExpr{
+			Type: t(""), // default type.
+			Expr: expr,
+		})
+	case okR:
+		var spec struct {
+			Expr string `spec:"expr"`
+			Type string `spec:"type"`
+		}
+		if err := asR.As(&spec); err != nil {
+			return err
+		}
+		c.Attrs = append(c.Attrs, &schema.GeneratedExpr{
+			Expr: spec.Expr,
+			Type: t(spec.Type),
+		})
+	}
+	return nil
+}
+
 func toValue(expr schema.Expr) (schemaspec.Value, error) {
 	var (
 		v   string
@@ -420,40 +453,75 @@ func normalizeQuotes(s string) (string, error) {
 }
 
 // FromIndex converts schema.Index to sqlspec.Index.
-func FromIndex(idx *schema.Index) (*sqlspec.Index, error) {
-	c := make([]*schemaspec.Ref, 0, len(idx.Parts))
-	for _, p := range idx.Parts {
-		if p.C == nil {
-			return nil, errors.New("index expression is not supported")
+func FromIndex(idx *schema.Index, partFns ...func(*schema.IndexPart, *sqlspec.IndexPart)) (*sqlspec.Index, error) {
+	spec := &sqlspec.Index{Name: idx.Name, Unique: idx.Unique}
+	convertCommentFromSchema(idx.Attrs, &spec.Extra.Attrs)
+	if parts, ok := columnsOnly(idx); ok {
+		spec.Columns = parts
+		return spec, nil
+	}
+	spec.Parts = make([]*sqlspec.IndexPart, len(idx.Parts))
+	for i, p := range idx.Parts {
+		part := &sqlspec.IndexPart{Desc: p.Desc}
+		switch {
+		case p.C == nil && p.X == nil:
+			return nil, fmt.Errorf("missing column or expression for key part of index %q", idx.Name)
+		case p.C != nil && p.X != nil:
+			return nil, fmt.Errorf("multiple key part definitions for index %q", idx.Name)
+		case p.C != nil:
+			part.Column = ColumnRef(p.C.Name)
+		case p.X != nil:
+			x, ok := p.X.(*schema.RawExpr)
+			if !ok {
+				return nil, fmt.Errorf("unexpected expression %T for index %q", p.X, idx.Name)
+			}
+			part.Expr = x.X
 		}
-		c = append(c, colRef(p.C.Name, idx.Table.Name))
+		for _, f := range partFns {
+			f(p, part)
+		}
+		spec.Parts[i] = part
 	}
-	i := &sqlspec.Index{
-		Name:    idx.Name,
-		Unique:  idx.Unique,
-		Columns: c,
+	return spec, nil
+}
+
+func columnsOnly(idx *schema.Index) ([]*schemaspec.Ref, bool) {
+	parts := make([]*schemaspec.Ref, len(idx.Parts))
+	for i, p := range idx.Parts {
+		if p.C == nil || p.Desc {
+			return nil, false
+		}
+		parts[i] = ColumnRef(p.C.Name)
 	}
-	convertCommentFromSchema(idx.Attrs, &i.Extra.Attrs)
-	return i, nil
+	return parts, true
 }
 
 // FromForeignKey converts schema.ForeignKey to sqlspec.ForeignKey.
 func FromForeignKey(s *schema.ForeignKey) (*sqlspec.ForeignKey, error) {
 	c := make([]*schemaspec.Ref, 0, len(s.Columns))
 	for _, v := range s.Columns {
-		c = append(c, colRef(v.Name, s.Table.Name))
+		c = append(c, ColumnRef(v.Name))
 	}
 	r := make([]*schemaspec.Ref, 0, len(s.RefColumns))
 	for _, v := range s.RefColumns {
-		r = append(r, colRef(v.Name, s.RefTable.Name))
+		ref := ColumnRef(v.Name)
+		if s.Table != s.RefTable {
+			ref = externalColRef(v.Name, s.RefTable.Name)
+		}
+		r = append(r, ref)
 	}
-	return &sqlspec.ForeignKey{
+	fk := &sqlspec.ForeignKey{
 		Symbol:     s.Symbol,
 		Columns:    c,
 		RefColumns: r,
-		OnDelete:   s.OnDelete,
-		OnUpdate:   s.OnUpdate,
-	}, nil
+	}
+	if s.OnUpdate != "" {
+		fk.OnUpdate = &schemaspec.Ref{V: Var(string(s.OnUpdate))}
+	}
+	if s.OnDelete != "" {
+		fk.OnDelete = &schemaspec.Ref{V: Var(string(s.OnDelete))}
+	}
+	return fk, nil
 }
 
 // FromCheck converts schema.Check to sqlspec.Check.
@@ -466,39 +534,101 @@ func FromCheck(s *schema.Check) *sqlspec.Check {
 
 // SchemaName returns the name from a ref to a schema.
 func SchemaName(ref *schemaspec.Ref) (string, error) {
+	if ref == nil {
+		return "", errors.New("missing 'schema' attribute")
+	}
 	parts := strings.Split(ref.V, ".")
 	if len(parts) < 2 || parts[0] != "$schema" {
-		return "", fmt.Errorf("expected ref format of $schema.name")
+		return "", errors.New("expected ref format of $schema.name")
 	}
 	return parts[1], nil
 }
 
-func columnName(ref *schemaspec.Ref) (string, error) {
+// ColumnByRef returns a column from the table by its reference.
+func ColumnByRef(t *schema.Table, ref *schemaspec.Ref) (*schema.Column, error) {
 	s := strings.Split(ref.V, "$column.")
 	if len(s) != 2 {
-		return "", fmt.Errorf("sqlspec: failed to extract column name from %q", ref)
+		return nil, fmt.Errorf("specutil: failed to extract column name from %q", ref)
 	}
-	return s[1], nil
+	c, ok := t.Column(s[1])
+	if !ok {
+		return nil, fmt.Errorf("specutil: unknown column %q in table %q", s[1], t.Name)
+	}
+	return c, nil
 }
 
-func tableName(ref *schemaspec.Ref) (string, error) {
+func externalRef(ref *schemaspec.Ref, sch *schema.Schema) (*schema.Table, *schema.Column, error) {
+	tbl, err := findTable(ref, sch)
+	if err != nil {
+		return nil, nil, err
+	}
+	c, err := ColumnByRef(tbl, ref)
+	if err != nil {
+		return nil, nil, err
+	}
+	return tbl, c, nil
+}
+
+// findTable finds the table referenced by ref in the provided schema. If the table
+// is not in the provided schema.Schema other schemas in the connected schema.Realm
+// are searched as well.
+func findTable(ref *schemaspec.Ref, sch *schema.Schema) (*schema.Table, error) {
+	qualifier, tblName, err := tableName(ref)
+	if err != nil {
+		return nil, err
+	}
+	// Search the same schema.
+	if qualifier == "" || qualifier == sch.Name {
+		tbl, ok := sch.Table(tblName)
+		if !ok {
+			return tbl, fmt.Errorf("sqlspec: table %q not found", tblName)
+		}
+		return tbl, nil
+	}
+	if sch.Realm == nil {
+		return nil, fmt.Errorf("sqlspec: table %s.%s not found", qualifier, tblName)
+	}
+	// Search for the table in another schemas in the realm.
+	sch, ok := sch.Realm.Schema(qualifier)
+	if !ok {
+		return nil, fmt.Errorf("sqlspec: schema %q not found", qualifier)
+	}
+	tbl, ok := sch.Table(tblName)
+	if !ok {
+		return tbl, fmt.Errorf("sqlspec: table %q not found", tblName)
+	}
+	return tbl, nil
+}
+
+func tableName(ref *schemaspec.Ref) (qualifier, name string, err error) {
 	s := strings.Split(ref.V, "$column.")
 	if len(s) != 2 {
-		return "", fmt.Errorf("sqlspec: failed to split by column name from %q", ref)
-
+		return "", "", fmt.Errorf("sqlspec: failed to split by column name from %q", ref)
 	}
-	s = strings.Split(s[0], ".")
-	if len(s) != 3 {
-		return "", fmt.Errorf("sqlspec: failed to extract table name from %q", s)
+	table := strings.TrimSuffix(s[0], ".")
+	s = strings.Split(table, ".")
+	switch len(s) {
+	case 2:
+		name = s[1]
+	case 3:
+		qualifier, name = s[1], s[2]
+	default:
+		return "", "", fmt.Errorf("sqlspec: failed to extract table name from %q", s)
 	}
-	return s[1], nil
+	return
 }
 
-func colRef(cName string, tName string) *schemaspec.Ref {
-	v := "$table." + tName + ".$column." + cName
-	return &schemaspec.Ref{
-		V: v,
-	}
+func isLocalRef(r *schemaspec.Ref) bool {
+	return strings.HasPrefix(r.V, "$column")
+}
+
+// ColumnRef returns the reference of a column by its name.
+func ColumnRef(cName string) *schemaspec.Ref {
+	return &schemaspec.Ref{V: "$column." + cName}
+}
+
+func externalColRef(cName string, tName string) *schemaspec.Ref {
+	return &schemaspec.Ref{V: "$table." + tName + ".$column." + cName}
 }
 
 // SchemaRef returns the schemaspec.Ref to the schema with the given name.
@@ -530,3 +660,20 @@ func convertCommentFromSchema(src []schema.Attr, trgt *[]*schemaspec.Attr) {
 		*trgt = append(*trgt, StrAttr("comment", c.Text))
 	}
 }
+
+// ReferenceVars holds the HCL variables
+// for foreign keys' referential-actions.
+var ReferenceVars = []string{
+	Var(string(schema.NoAction)),
+	Var(string(schema.Restrict)),
+	Var(string(schema.Cascade)),
+	Var(string(schema.SetNull)),
+	Var(string(schema.SetDefault)),
+}
+
+// Var formats a string as variable to make it HCL compatible.
+// The result is simple, replace each space with underscore.
+func Var(s string) string { return strings.ReplaceAll(s, " ", "_") }
+
+// FromVar is the inverse function of Var.
+func FromVar(s string) string { return strings.ReplaceAll(s, "_", " ") }
