@@ -28,7 +28,10 @@ func FormatType(t schema.Type) (string, error) {
 			f = fmt.Sprintf("%s(1)", f)
 		}
 	case *schema.BoolType:
-		f = strings.ToLower(t.T)
+		// BOOLEAN can be abbreviated as BOOL.
+		if f = strings.ToLower(t.T); f == TypeBool {
+			f = TypeBoolean
+		}
 	case *schema.BinaryType:
 		f = strings.ToLower(t.T)
 	case *CurrencyType:
@@ -70,16 +73,9 @@ func FormatType(t schema.Type) (string, error) {
 			return "", fmt.Errorf("postgres: unexpected string type: %q", t.T)
 		}
 	case *schema.TimeType:
-		switch f = strings.ToLower(t.T); f {
-		// TIMESTAMPTZ is accepted as an abbreviation for TIMESTAMP WITH TIME ZONE.
-		case TypeTimestampTZ:
-			f = TypeTimestampWTZ
-		// TIME be equivalent to TIME WITHOUT TIME ZONE.
-		case TypeTime:
-			f = TypeTimeWOTZ
-		// TIMESTAMP be equivalent to TIMESTAMP WITHOUT TIME ZONE.
-		case TypeTimestamp:
-			f = TypeTimestampWOTZ
+		f = timeAlias(t.T)
+		if p := t.Precision; p != nil && *p != defaultTimePrecision && strings.HasPrefix(f, "time") {
+			f += fmt.Sprintf("(%d)", *p)
 		}
 	case *schema.FloatType:
 		switch f = strings.ToLower(t.T); f {
@@ -138,15 +134,6 @@ func FormatType(t schema.Type) (string, error) {
 	return f, nil
 }
 
-// mustFormat calls to FormatType and panics in case of error.
-func mustFormat(t schema.Type) string {
-	s, err := FormatType(t)
-	if err != nil {
-		panic(err)
-	}
-	return s
-}
-
 // ParseType returns the schema.Type value represented by the given raw type.
 // The raw value is expected to follow the format in PostgreSQL information schema
 // or as an input for the CREATE TABLE statement.
@@ -161,7 +148,7 @@ func ParseType(typ string) (schema.Type, error) {
 		d = &columnDesc{typ: TypeArray, udt: t}
 	}
 	t := columnType(d)
-	// If the type is unknown (to us), we fallback to user-defined but expect
+	// If the type is unknown (to us), we fall back to user-defined but expect
 	// to improve this in future versions by ensuring this against the database.
 	if ut, ok := t.(*schema.UnsupportedType); ok {
 		t = &UserDefinedType{T: ut.T}
@@ -184,16 +171,19 @@ func arrayType(t string) (string, bool) {
 
 // columnDesc represents a column descriptor.
 type columnDesc struct {
-	typ       string
-	size      int64
-	udt       string
-	precision int64
-	scale     int64
-	typtype   string
-	typid     int64
-	parts     []string
+	typ           string
+	size          int64
+	udt           string
+	precision     int64
+	timePrecision *int64
+	scale         int64
+	typtype       string
+	typid         int64
+	parts         []string
 }
 
+var reDigits = regexp.MustCompile(`\d`)
+
 func parseColumn(s string) (*columnDesc, error) {
 	parts := strings.FieldsFunc(s, func(r rune) bool {
 		return r == '(' || r == ')' || r == ' ' || r == ','
@@ -231,6 +221,21 @@ func parseColumn(s string) (*columnDesc, error) {
 		c.precision = 53
 	case TypeReal, TypeFloat4:
 		c.precision = 24
+	case TypeTime, TypeTimeTZ, TypeTimestamp, TypeTimestampTZ:
+		t, p := s, int64(defaultTimePrecision)
+		// If the second part is only one digit it is the precision argument.
+		// For cases like "timestamp(4) with time zone" make sure to not drop
+		// the rest of the type definition.
+		if len(parts) > 1 && reDigits.MatchString(parts[1]) {
+			i, err := strconv.ParseInt(parts[1], 10, 64)
+			if err != nil {
+				return nil, fmt.Errorf("postgres: parse time precision %q: %w", parts[1], err)
+			}
+			p = i
+			t = strings.Join(append(c.parts[:1], c.parts[2:]...), " ")
+		}
+		c.typ = timeAlias(t)
+		c.timePrecision = &p
 	default:
 		c.typ = s
 	}
@@ -280,3 +285,22 @@ func parseBitParts(parts []string, c *columnDesc) error {
 	c.size = size
 	return nil
 }
+
+// timeAlias returns the abbreviation for the given time type.
+func timeAlias(t string) string {
+	switch t = strings.ToLower(t); t {
+	// TIMESTAMPTZ be equivalent to TIMESTAMP WITH TIME ZONE.
+	case TypeTimestampWTZ:
+		t = TypeTimestampTZ
+	// TIMESTAMP be equivalent to TIMESTAMP WITHOUT TIME ZONE.
+	case TypeTimestampWOTZ:
+		t = TypeTimestamp
+	// TIME be equivalent to TIME WITHOUT TIME ZONE.
+	case TypeTimeWOTZ:
+		t = TypeTime
+	// TIMETZ be equivalent to TIME WITH TIME ZONE.
+	case TypeTimeWTZ:
+		t = TypeTimeTZ
+	}
+	return t
+}
