@@ -26,159 +26,158 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 	if err != nil {
 		return nil, err
 	}
-	realm := &schema.Realm{Schemas: schemas, Attrs: []schema.Attr{&schema.Collation{V: i.collate}, &CType{V: i.ctype}}}
-	for _, s := range schemas {
-		names, err := i.tableNames(ctx, s.Name, nil)
-		if err != nil {
-			return nil, err
-		}
-		for _, name := range names {
-			t, err := i.inspectTable(ctx, name, &schema.InspectTableOptions{Schema: s.Name}, s)
-			if err != nil {
-				return nil, err
-			}
-			s.Tables = append(s.Tables, t)
-		}
-		s.Realm = realm
+	r := schema.NewRealm(schemas...).SetCollation(i.collate)
+	r.Attrs = append(r.Attrs, &CType{V: i.ctype})
+	if len(schemas) == 0 || !sqlx.ModeInspectRealm(opts).Is(schema.InspectTables) {
+		return r, nil
+	}
+	if err := i.inspectTables(ctx, r, nil); err != nil {
+		return nil, err
 	}
 	sqlx.LinkSchemaTables(schemas)
-	return realm, nil
+	return r, nil
 }
 
 // InspectSchema returns schema descriptions of the tables in the given schema.
 // If the schema name is empty, the result will be the attached schema.
 func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.InspectOptions) (s *schema.Schema, err error) {
-	var schemas []*schema.Schema
-	switch name {
-	case "":
-		rows, err := i.QueryContext(ctx, "SELECT CURRENT_SCHEMA()")
-		if err != nil {
-			return nil, fmt.Errorf("postgres: query attached schema: %w", err)
-		}
-		if err := sqlx.ScanOne(rows, &name); err != nil {
-			return nil, fmt.Errorf("postgres: scan attached schema: %w", err)
-		}
-		schemas = append(schemas, &schema.Schema{Name: name})
-	default:
-		if schemas, err = i.schemas(ctx, &schema.InspectRealmOption{Schemas: []string{name}}); err != nil {
-			return nil, err
-		}
-		if len(schemas) == 0 {
-			return nil, &schema.NotExistError{
-				Err: fmt.Errorf("postgres: schema %q was not found", name),
-			}
-		}
-	}
-	names, err := i.tableNames(ctx, name, opts)
+	schemas, err := i.schemas(ctx, &schema.InspectRealmOption{Schemas: []string{name}})
 	if err != nil {
 		return nil, err
 	}
-	s = schemas[0]
-	for _, name := range names {
-		t, err := i.inspectTable(ctx, name, &schema.InspectTableOptions{Schema: s.Name}, s)
-		if err != nil {
+	switch n := len(schemas); {
+	case n == 0:
+		return nil, &schema.NotExistError{Err: fmt.Errorf("postgres: schema %q was not found", name)}
+	case n > 1:
+		return nil, fmt.Errorf("postgres: %d schemas were found for %q", n, name)
+	}
+	r := schema.NewRealm(schemas...).SetCollation(i.collate)
+	r.Attrs = append(r.Attrs, &CType{V: i.ctype})
+	if sqlx.ModeInspectSchema(opts).Is(schema.InspectTables) {
+		if err := i.inspectTables(ctx, r, opts); err != nil {
 			return nil, err
 		}
-		s.Tables = append(s.Tables, t)
+		sqlx.LinkSchemaTables(schemas)
 	}
-	sqlx.LinkSchemaTables(schemas)
-	s.Realm = &schema.Realm{Schemas: schemas, Attrs: []schema.Attr{&schema.Collation{V: i.collate}, &CType{V: i.ctype}}}
-	return s, nil
+	return r.Schemas[0], nil
 }
 
-// InspectTable returns the schema description of the given table.
-func (i *inspect) InspectTable(ctx context.Context, name string, opts *schema.InspectTableOptions) (*schema.Table, error) {
-	return i.inspectTable(ctx, name, opts, nil)
-}
-
-func (i *inspect) inspectTable(ctx context.Context, name string, opts *schema.InspectTableOptions, top *schema.Schema) (*schema.Table, error) {
-	t, err := i.table(ctx, name, opts)
-	if err != nil {
-		return nil, err
+func (i *inspect) inspectTables(ctx context.Context, r *schema.Realm, opts *schema.InspectOptions) error {
+	if err := i.tables(ctx, r, opts); err != nil {
+		return err
 	}
-	if top != nil {
-		// Link the table to its top element if provided.
-		t.Schema = top
+	for _, s := range r.Schemas {
+		if len(s.Tables) == 0 {
+			continue
+		}
+		if err := i.columns(ctx, s); err != nil {
+			return err
+		}
+		if err := i.indexes(ctx, s); err != nil {
+			return err
+		}
+		if err := i.partitions(s); err != nil {
+			return err
+		}
+		if err := i.fks(ctx, s); err != nil {
+			return err
+		}
+		if err := i.checks(ctx, s); err != nil {
+			return err
+		}
 	}
-	if err := i.columns(ctx, t); err != nil {
-		return nil, err
-	}
-	if err := i.indexes(ctx, t); err != nil {
-		return nil, err
-	}
-	if err := i.fks(ctx, t); err != nil {
-		return nil, err
-	}
-	if err := i.checks(ctx, t); err != nil {
-		return nil, err
-	}
-	return t, nil
+	return nil
 }
 
 // table returns the table from the database, or a NotExistError if the table was not found.
-func (i *inspect) table(ctx context.Context, name string, opts *schema.InspectTableOptions) (*schema.Table, error) {
+func (i *inspect) tables(ctx context.Context, realm *schema.Realm, opts *schema.InspectOptions) error {
 	var (
-		args  = []interface{}{name}
-		query = tableQuery
+		args  []interface{}
+		query = fmt.Sprintf(tablesQuery, nArgs(0, len(realm.Schemas)))
 	)
-	if opts != nil && opts.Schema != "" {
-		query = tableSchemaQuery
-		args = append(args, opts.Schema)
+	for _, s := range realm.Schemas {
+		args = append(args, s.Name)
 	}
-	var (
-		tSchema, comment sql.NullString
-		rows, err        = i.QueryContext(ctx, query, args...)
-	)
-	if err != nil {
-		return nil, err
-	}
-	if err := sqlx.ScanOne(rows, &tSchema, &comment); err != nil {
-		if err == sql.ErrNoRows {
-			return nil, &schema.NotExistError{
-				Err: fmt.Errorf("postgres: table %q was not found", name),
-			}
+	if opts != nil && len(opts.Tables) > 0 {
+		for _, t := range opts.Tables {
+			args = append(args, t)
 		}
-		return nil, err
+		query = fmt.Sprintf(tablesQueryArgs, nArgs(0, len(realm.Schemas)), nArgs(len(realm.Schemas), len(opts.Tables)))
 	}
-	t := &schema.Table{Name: name, Schema: &schema.Schema{Name: tSchema.String}}
-	if sqlx.ValidString(comment) {
-		t.Attrs = append(t.Attrs, &schema.Comment{
-			Text: comment.String,
-		})
-	}
-	return t, nil
-}
-
-// columns queries and appends the columns of the given table.
-func (i *inspect) columns(ctx context.Context, t *schema.Table) error {
-	rows, err := i.QueryContext(ctx, columnsQuery, t.Schema.Name, t.Name)
+	rows, err := i.QueryContext(ctx, query, args...)
 	if err != nil {
-		return fmt.Errorf("postgres: querying %q columns: %w", t.Name, err)
+		return err
 	}
 	defer rows.Close()
 	for rows.Next() {
-		if err := i.addColumn(t, rows); err != nil {
+		var tSchema, name, comment, partattrs, partstart, partexprs sql.NullString
+		if err := rows.Scan(&tSchema, &name, &comment, &partattrs, &partstart, &partexprs); err != nil {
+			return fmt.Errorf("scan table information: %w", err)
+		}
+		if !sqlx.ValidString(tSchema) || !sqlx.ValidString(name) {
+			return fmt.Errorf("invalid schema or table name: %q.%q", tSchema.String, name.String)
+		}
+		s, ok := realm.Schema(tSchema.String)
+		if !ok {
+			return fmt.Errorf("schema %q was not found in realm", tSchema.String)
+		}
+		t := &schema.Table{Name: name.String}
+		s.AddTables(t)
+		if sqlx.ValidString(comment) {
+			t.SetComment(comment.String)
+		}
+		if sqlx.ValidString(partattrs) {
+			t.AddAttrs(&Partition{
+				start: partstart.String,
+				attrs: partattrs.String,
+				exprs: partexprs.String,
+			})
+		}
+	}
+	return rows.Close()
+}
+
+// columns queries and appends the columns of the given table.
+func (i *inspect) columns(ctx context.Context, s *schema.Schema) error {
+	query := columnsQuery
+	if i.crdb {
+		query = crdbColumnsQuery
+	}
+	rows, err := i.querySchema(ctx, query, s)
+	if err != nil {
+		return fmt.Errorf("postgres: querying schema %q columns: %w", s.Name, err)
+	}
+	defer rows.Close()
+	for rows.Next() {
+		if err := i.addColumn(s, rows); err != nil {
 			return fmt.Errorf("postgres: %w", err)
 		}
 	}
 	if err := rows.Close(); err != nil {
 		return err
 	}
-	if err := i.enumValues(ctx, t.Columns); err != nil {
+	if err := i.enumValues(ctx, s); err != nil {
 		return err
 	}
 	return nil
 }
 
 // addColumn scans the current row and adds a new column from it to the table.
-func (i *inspect) addColumn(t *schema.Table, rows *sql.Rows) error {
+func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) error {
 	var (
-		typid, maxlen, precision, scale, seqstart, seqinc                                              sql.NullInt64
-		name, typ, nullable, defaults, udt, identity, generation, charset, collation, comment, typtype sql.NullString
+		typid, maxlen, precision, timeprecision, scale, seqstart, seqinc                                              sql.NullInt64
+		table, name, typ, nullable, defaults, udt, identity, genidentity, genexpr, charset, collate, comment, typtype sql.NullString
 	)
-	if err := rows.Scan(&name, &typ, &nullable, &defaults, &maxlen, &precision, &scale, &charset, &collation, &udt, &identity, &seqstart, &seqinc, &generation, &comment, &typtype, &typid); err != nil {
+	if err := rows.Scan(
+		&table, &name, &typ, &nullable, &defaults, &maxlen, &precision, &timeprecision, &scale, &charset,
+		&collate, &udt, &identity, &seqstart, &seqinc, &genidentity, &genexpr, &comment, &typtype, &typid,
+	); err != nil {
 		return err
 	}
+	t, ok := s.Table(table.String)
+	if !ok {
+		return fmt.Errorf("table %q was not found in schema", table.String)
+	}
 	c := &schema.Column{
 		Name: name.String,
 		Type: &schema.ColumnType{
@@ -187,40 +186,40 @@ func (i *inspect) addColumn(t *schema.Table, rows *sql.Rows) error {
 		},
 	}
 	c.Type.Type = columnType(&columnDesc{
-		typ:       typ.String,
-		size:      maxlen.Int64,
-		udt:       udt.String,
-		precision: precision.Int64,
-		scale:     scale.Int64,
-		typtype:   typtype.String,
-		typid:     typid.Int64,
+		typ:           typ.String,
+		size:          maxlen.Int64,
+		udt:           udt.String,
+		scale:         scale.Int64,
+		typtype:       typtype.String,
+		typid:         typid.Int64,
+		precision:     precision.Int64,
+		timePrecision: &timeprecision.Int64,
 	})
-	if sqlx.ValidString(defaults) {
+	if defaults.Valid {
 		c.Default = defaultExpr(c, defaults.String)
 	}
 	if identity.String == "YES" {
 		c.Attrs = append(c.Attrs, &Identity{
-			Generation: generation.String,
+			Generation: genidentity.String,
 			Sequence: &Sequence{
 				Start:     seqstart.Int64,
 				Increment: seqinc.Int64,
 			},
 		})
 	}
-	if sqlx.ValidString(comment) {
-		c.Attrs = append(c.Attrs, &schema.Comment{
-			Text: comment.String,
+	if sqlx.ValidString(genexpr) {
+		c.Attrs = append(c.Attrs, &schema.GeneratedExpr{
+			Expr: genexpr.String,
 		})
 	}
+	if sqlx.ValidString(comment) {
+		c.SetComment(comment.String)
+	}
 	if sqlx.ValidString(charset) {
-		c.Attrs = append(c.Attrs, &schema.Charset{
-			V: charset.String,
-		})
+		c.SetCharset(charset.String)
 	}
-	if sqlx.ValidString(collation) {
-		c.Attrs = append(c.Attrs, &schema.Collation{
-			V: collation.String,
-		})
+	if sqlx.ValidString(collate) {
+		c.SetCollation(collate.String)
 	}
 	t.Columns = append(t.Columns, c)
 	return nil
@@ -229,7 +228,7 @@ func (i *inspect) addColumn(t *schema.Table, rows *sql.Rows) error {
 func columnType(c *columnDesc) schema.Type {
 	var typ schema.Type
 	switch t := c.typ; strings.ToLower(t) {
-	case TypeBigInt, TypeInt8, TypeInt, TypeInteger, TypeInt4, TypeSmallInt, TypeInt2:
+	case TypeBigInt, TypeInt8, TypeInt, TypeInteger, TypeInt4, TypeSmallInt, TypeInt2, TypeInt64:
 		typ = &schema.IntegerType{T: t}
 	case TypeBit, TypeBitVar:
 		typ = &BitType{T: t, Len: c.size}
@@ -243,11 +242,17 @@ func columnType(c *columnDesc) schema.Type {
 		typ = &schema.StringType{T: t, Size: int(c.size)}
 	case TypeCIDR, TypeInet, TypeMACAddr, TypeMACAddr8:
 		typ = &NetworkType{T: t}
-	case TypeCircle, TypeLine, TypeLseg, TypeBox, TypePath, TypePolygon, TypePoint:
+	case TypeCircle, TypeLine, TypeLseg, TypeBox, TypePath, TypePolygon, TypePoint, TypeGeometry:
 		typ = &schema.SpatialType{T: t}
-	case TypeDate, TypeTime, TypeTimeWTZ, TypeTimeWOTZ,
-		TypeTimestamp, TypeTimestampTZ, TypeTimestampWTZ, TypeTimestampWOTZ:
+	case TypeDate:
 		typ = &schema.TimeType{T: t}
+	case TypeTime, TypeTimeWOTZ, TypeTimeTZ, TypeTimeWTZ, TypeTimestamp,
+		TypeTimestampTZ, TypeTimestampWTZ, TypeTimestampWOTZ:
+		p := defaultTimePrecision
+		if c.timePrecision != nil {
+			p = int(*c.timePrecision)
+		}
+		typ = &schema.TimeType{T: t, Precision: &p}
 	case TypeInterval:
 		// TODO: get 'interval_type' from query above before implementing.
 		typ = &schema.UnsupportedType{T: t}
@@ -286,33 +291,31 @@ func columnType(c *columnDesc) schema.Type {
 }
 
 // enumValues fills enum columns with their values from the database.
-func (i *inspect) enumValues(ctx context.Context, columns []*schema.Column) error {
+func (i *inspect) enumValues(ctx context.Context, s *schema.Schema) error {
 	var (
 		args  []interface{}
 		ids   = make(map[int64][]*schema.EnumType)
-		query = "SELECT enumtypid, enumlabel FROM pg_enum WHERE enumtypid IN ("
+		query = "SELECT enumtypid, enumlabel FROM pg_enum WHERE enumtypid IN (%s)"
 	)
-	for _, c := range columns {
-		if enum, ok := c.Type.Type.(*enumType); ok {
-			if _, ok := ids[enum.ID]; !ok {
-				if len(args) > 0 {
-					query += ", "
+	for _, t := range s.Tables {
+		for _, c := range t.Columns {
+			if enum, ok := c.Type.Type.(*enumType); ok {
+				if _, ok := ids[enum.ID]; !ok {
+					args = append(args, enum.ID)
 				}
-				args = append(args, enum.ID)
-				query += fmt.Sprintf("$%d", len(args))
+				// Convert the intermediate type to the
+				// standard schema.EnumType.
+				e := &schema.EnumType{T: enum.T}
+				c.Type.Type = e
+				c.Type.Raw = enum.T
+				ids[enum.ID] = append(ids[enum.ID], e)
 			}
-			// Convert the intermediate type to the standard schema.enumType.
-			e := &schema.EnumType{T: enum.T}
-			c.Type.Type = e
-			c.Type.Raw = enum.T
-			ids[enum.ID] = append(ids[enum.ID], e)
 		}
 	}
-	query += ")"
 	if len(ids) == 0 {
 		return nil
 	}
-	rows, err := i.QueryContext(ctx, query, args...)
+	rows, err := i.QueryContext(ctx, fmt.Sprintf(query, nArgs(0, len(args))), args...)
 	if err != nil {
 		return fmt.Errorf("postgres: querying enum values: %w", err)
 	}
@@ -333,30 +336,37 @@ func (i *inspect) enumValues(ctx context.Context, columns []*schema.Column) erro
 }
 
 // indexes queries and appends the indexes of the given table.
-func (i *inspect) indexes(ctx context.Context, t *schema.Table) error {
-	rows, err := i.QueryContext(ctx, indexesQuery, t.Schema.Name, t.Name)
+func (i *inspect) indexes(ctx context.Context, s *schema.Schema) error {
+	if i.conn.crdb {
+		return i.crdbIndexes(ctx, s)
+	}
+	rows, err := i.querySchema(ctx, indexesQuery, s)
 	if err != nil {
-		return fmt.Errorf("postgres: querying %q indexes: %w", t.Name, err)
+		return fmt.Errorf("postgres: querying schema %q indexes: %w", s.Name, err)
 	}
 	defer rows.Close()
-	if err := i.addIndexes(t, rows); err != nil {
+	if err := i.addIndexes(s, rows); err != nil {
 		return err
 	}
 	return rows.Err()
 }
 
 // addIndexes scans the rows and adds the indexes to the table.
-func (i *inspect) addIndexes(t *schema.Table, rows *sql.Rows) error {
+func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 	names := make(map[string]*schema.Index)
 	for rows.Next() {
 		var (
-			name, typ                            string
-			uniq, primary                        bool
-			asc, desc, nullsfirst, nullslast     sql.NullBool
-			column, contype, pred, expr, comment sql.NullString
+			uniq, primary                                 bool
+			table, name, typ                              string
+			desc, nullsfirst, nullslast                   sql.NullBool
+			column, contype, pred, expr, comment, options sql.NullString
 		)
-		if err := rows.Scan(&name, &typ, &column, &primary, &uniq, &contype, &pred, &expr, &asc, &desc, &nullsfirst, &nullslast, &comment); err != nil {
-			return fmt.Errorf("postgres: scanning index: %w", err)
+		if err := rows.Scan(&table, &name, &typ, &column, &primary, &uniq, &contype, &pred, &expr, &desc, &nullsfirst, &nullslast, &comment, &options); err != nil {
+			return fmt.Errorf("postgres: scanning indexes for schema %q: %w", s.Name, err)
+		}
+		t, ok := s.Table(table)
+		if !ok {
+			return fmt.Errorf("table %q was not found in schema", table)
 		}
 		idx, ok := names[name]
 		if !ok {
@@ -377,6 +387,13 @@ func (i *inspect) addIndexes(t *schema.Table, rows *sql.Rows) error {
 			if sqlx.ValidString(pred) {
 				idx.Attrs = append(idx.Attrs, &IndexPredicate{P: pred.String})
 			}
+			if sqlx.ValidString(options) {
+				p, err := newIndexStorage(options.String)
+				if err != nil {
+					return err
+				}
+				idx.Attrs = append(idx.Attrs, p)
+			}
 			names[name] = idx
 			if primary {
 				t.PrimaryKey = idx
@@ -384,28 +401,24 @@ func (i *inspect) addIndexes(t *schema.Table, rows *sql.Rows) error {
 				t.Indexes = append(t.Indexes, idx)
 			}
 		}
-		part := &schema.IndexPart{
-			SeqNo: len(idx.Parts) + 1,
-			Attrs: []schema.Attr{
-				&IndexColumnProperty{
-					Asc:        asc.Bool,
-					Desc:       desc.Bool,
-					NullsFirst: nullsfirst.Bool,
-					NullsLast:  nullslast.Bool,
-				},
-			},
+		part := &schema.IndexPart{SeqNo: len(idx.Parts) + 1, Desc: desc.Bool}
+		if nullsfirst.Bool || nullslast.Bool {
+			part.Attrs = append(part.Attrs, &IndexColumnProperty{
+				NullsFirst: nullsfirst.Bool,
+				NullsLast:  nullslast.Bool,
+			})
 		}
 		switch {
-		case sqlx.ValidString(expr):
-			part.X = &schema.RawExpr{
-				X: expr.String,
-			}
 		case sqlx.ValidString(column):
 			part.C, ok = t.Column(column.String)
 			if !ok {
 				return fmt.Errorf("postgres: column %q was not found for index %q", column.String, idx.Name)
 			}
 			part.C.Indexes = append(part.C.Indexes, idx)
+		case sqlx.ValidString(expr):
+			part.X = &schema.RawExpr{
+				X: expr.String,
+			}
 		default:
 			return fmt.Errorf("postgres: invalid part for index %q", idx.Name)
 		}
@@ -414,43 +427,97 @@ func (i *inspect) addIndexes(t *schema.Table, rows *sql.Rows) error {
 	return nil
 }
 
+// partitions builds the partition each table in the schema.
+func (i *inspect) partitions(s *schema.Schema) error {
+	for _, t := range s.Tables {
+		var d Partition
+		if !sqlx.Has(t.Attrs, &d) {
+			continue
+		}
+		switch s := strings.ToLower(d.start); s {
+		case "r":
+			d.T = PartitionTypeRange
+		case "l":
+			d.T = PartitionTypeList
+		case "h":
+			d.T = PartitionTypeHash
+		default:
+			return fmt.Errorf("postgres: unexpected partition strategy %q", s)
+		}
+		idxs := strings.Split(strings.TrimSpace(d.attrs), " ")
+		if len(idxs) == 0 {
+			return fmt.Errorf("postgres: no columns/expressions were found in partition key for column %q", t.Name)
+		}
+		for i := range idxs {
+			switch idx, err := strconv.Atoi(idxs[i]); {
+			case err != nil:
+				return fmt.Errorf("postgres: faild parsing partition key index %q", idxs[i])
+			// An expression.
+			case idx == 0:
+				j := sqlx.ExprLastIndex(d.exprs)
+				if j == -1 {
+					return fmt.Errorf("postgres: no expression found in partition key: %q", d.exprs)
+				}
+				d.Parts = append(d.Parts, &PartitionPart{
+					X: &schema.RawExpr{X: d.exprs[:j+1]},
+				})
+				d.exprs = strings.TrimPrefix(d.exprs[j+1:], ", ")
+			// A column at index idx-1.
+			default:
+				if idx > len(t.Columns) {
+					return fmt.Errorf("postgres: unexpected column index %d", idx)
+				}
+				d.Parts = append(d.Parts, &PartitionPart{
+					C: t.Columns[idx-1],
+				})
+			}
+		}
+		schema.ReplaceOrAppend(&t.Attrs, &d)
+	}
+	return nil
+}
+
 // fks queries and appends the foreign keys of the given table.
-func (i *inspect) fks(ctx context.Context, t *schema.Table) error {
-	rows, err := i.QueryContext(ctx, fksQuery, t.Schema.Name, t.Name)
+func (i *inspect) fks(ctx context.Context, s *schema.Schema) error {
+	rows, err := i.querySchema(ctx, fksQuery, s)
 	if err != nil {
-		return fmt.Errorf("postgres: querying %q foreign keys: %w", t.Name, err)
+		return fmt.Errorf("postgres: querying schema %q foreign keys: %w", s.Name, err)
 	}
 	defer rows.Close()
-	if err := sqlx.ScanFKs(t, rows); err != nil {
+	if err := sqlx.SchemaFKs(s, rows); err != nil {
 		return fmt.Errorf("postgres: %w", err)
 	}
 	return rows.Err()
 }
 
 // checks queries and appends the check constraints of the given table.
-func (i *inspect) checks(ctx context.Context, t *schema.Table) error {
-	rows, err := i.QueryContext(ctx, checksQuery, t.Schema.Name, t.Name)
+func (i *inspect) checks(ctx context.Context, s *schema.Schema) error {
+	rows, err := i.querySchema(ctx, checksQuery, s)
 	if err != nil {
-		return fmt.Errorf("mysql: querying %q check constraints: %w", t.Name, err)
+		return fmt.Errorf("postgres: querying schema %q check constraints: %w", s.Name, err)
 	}
 	defer rows.Close()
-	if err := i.addChecks(t, rows); err != nil {
+	if err := i.addChecks(s, rows); err != nil {
 		return err
 	}
 	return rows.Err()
 }
 
 // addChecks scans the rows and adds the checks to the table.
-func (i *inspect) addChecks(t *schema.Table, rows *sql.Rows) error {
+func (i *inspect) addChecks(s *schema.Schema, rows *sql.Rows) error {
 	names := make(map[string]*schema.Check)
 	for rows.Next() {
 		var (
-			noInherit                     bool
-			name, column, clause, indexes string
+			noInherit                            bool
+			table, name, column, clause, indexes string
 		)
-		if err := rows.Scan(&name, &clause, &column, &indexes, &noInherit); err != nil {
+		if err := rows.Scan(&table, &name, &clause, &column, &indexes, &noInherit); err != nil {
 			return fmt.Errorf("postgres: scanning check: %w", err)
 		}
+		t, ok := s.Table(table)
+		if !ok {
+			return fmt.Errorf("table %q was not found in schema", table)
+		}
 		if _, ok := t.Column(column); !ok {
 			return fmt.Errorf("postgres: column %q was not found for check %q", column, name)
 		}
@@ -475,8 +542,19 @@ func (i *inspect) schemas(ctx context.Context, opts *schema.InspectRealmOption)
 		args  []interface{}
 		query = schemasQuery
 	)
-	if opts != nil && len(opts.Schemas) > 0 {
-		query, args = inStrings(opts.Schemas, schemasQueryArgs, args)
+	if opts != nil {
+		switch n := len(opts.Schemas); {
+		case n == 1 && opts.Schemas[0] == "":
+			query = fmt.Sprintf(schemasQueryArgs, "= CURRENT_SCHEMA()")
+		case n == 1 && opts.Schemas[0] != "":
+			query = fmt.Sprintf(schemasQueryArgs, "= $1")
+			args = append(args, opts.Schemas[0])
+		case n > 0:
+			query = fmt.Sprintf(schemasQueryArgs, "IN ("+nArgs(0, len(opts.Schemas))+")")
+			for _, s := range opts.Schemas {
+				args = append(args, s)
+			}
+		}
 	}
 	rows, err := i.QueryContext(ctx, query, args...)
 	if err != nil {
@@ -493,46 +571,30 @@ func (i *inspect) schemas(ctx context.Context, opts *schema.InspectRealmOption)
 			Name: name,
 		})
 	}
+	if err := rows.Close(); err != nil {
+		return nil, err
+	}
 	return schemas, nil
 }
 
-// tableNames returns a list of all tables exist in the schema.
-func (i *inspect) tableNames(ctx context.Context, schema string, opts *schema.InspectOptions) ([]string, error) {
-	query, args := tablesQuery, []interface{}{schema}
-	if opts != nil && len(opts.Tables) > 0 {
-		query, args = inStrings(opts.Tables, tablesQueryArgs, args)
+func (i *inspect) querySchema(ctx context.Context, query string, s *schema.Schema) (*sql.Rows, error) {
+	args := []interface{}{s.Name}
+	for _, t := range s.Tables {
+		args = append(args, t.Name)
 	}
-	rows, err := i.QueryContext(ctx, query, args...)
-	if err != nil {
-		return nil, fmt.Errorf("postgres: querying schema tables: %w", err)
-	}
-	names, err := sqlx.ScanStrings(rows)
-	if err != nil {
-		return nil, fmt.Errorf("postgres: scanning table names: %w", err)
-	}
-	return names, nil
+	return i.QueryContext(ctx, fmt.Sprintf(query, nArgs(1, len(s.Tables))), args...)
 }
 
-func inStrings(s []string, query string, args []interface{}) (string, []interface{}) {
+func nArgs(start, n int) string {
 	var b strings.Builder
-	switch len(s) {
-	case 1:
-		args = append(args, s[0])
-		b.WriteString("= $")
-		b.WriteString(strconv.Itoa(len(args)))
-	default:
-		b.WriteString("IN (")
-		for i := range s {
-			args = append(args, s[i])
-			if i > 0 {
-				b.WriteString(", ")
-			}
-			b.WriteByte('$')
-			b.WriteString(strconv.Itoa(len(args)))
+	for i := 1; i <= n; i++ {
+		if i > 1 {
+			b.WriteString(", ")
 		}
-		b.WriteByte(')')
+		b.WriteByte('$')
+		b.WriteString(strconv.Itoa(start + i))
 	}
-	return fmt.Sprintf(query, b.String()), args
+	return b.String()
 }
 
 func defaultExpr(c *schema.Column, x string) schema.Expr {
@@ -682,10 +744,21 @@ type (
 	// https://www.postgresql.org/docs/current/functions-info.html#FUNCTIONS-INFO-INDEX-COLUMN-PROPS
 	IndexColumnProperty struct {
 		schema.Attr
-		Asc        bool
-		Desc       bool
+		// NullsFirst defaults to true for DESC indexes.
 		NullsFirst bool
-		NullsLast  bool
+		// NullsLast defaults to true for ASC indexes.
+		NullsLast bool
+	}
+
+	// IndexStorageParams describes index storage parameters add with the WITH clause.
+	// https://www.postgresql.org/docs/current/sql-createindex.html#SQL-CREATEINDEX-STORAGE-PARAMETERS
+	IndexStorageParams struct {
+		schema.Attr
+		// AutoSummarize defines the authsummarize storage parameter.
+		AutoSummarize bool
+		// PagesPerRange defines pages_per_range storage
+		// parameter for BRIN indexes. Defaults to 128.
+		PagesPerRange int64
 	}
 
 	// NoInherit attribute defines the NO INHERIT flag for CHECK constraint.
@@ -701,60 +774,119 @@ type (
 		schema.Attr
 		Columns []string
 	}
+
+	// Partition defines the spec of a partitioned table.
+	Partition struct {
+		schema.Attr
+		// T defines the type/strategy of the partition.
+		// Can be one of: RANGE, LIST, HASH.
+		T string
+		// Partition parts. The additional attributes
+		// on each part can be used to control collation.
+		Parts []*PartitionPart
+
+		// Internal info returned from pg_partitioned_table.
+		start, attrs, exprs string
+	}
+
+	// An PartitionPart represents an index part that
+	// can be either an expression or a column.
+	PartitionPart struct {
+		X     schema.Expr
+		C     *schema.Column
+		Attrs []schema.Attr
+	}
 )
 
+// newIndexStorage parses and returns the index storage parameters.
+func newIndexStorage(opts string) (*IndexStorageParams, error) {
+	params := &IndexStorageParams{}
+	for _, p := range strings.Split(strings.Trim(opts, "{}"), ",") {
+		kv := strings.Split(p, "=")
+		if len(kv) != 2 {
+			return nil, fmt.Errorf("invalid index storage parameter: %s", p)
+		}
+		switch kv[0] {
+		case "autosummarize":
+			b, err := strconv.ParseBool(kv[1])
+			if err != nil {
+				return nil, fmt.Errorf("failed parsing autosummarize %q: %w", kv[1], err)
+			}
+			params.AutoSummarize = b
+		case "pages_per_range":
+			i, err := strconv.ParseInt(kv[1], 10, 64)
+			if err != nil {
+				return nil, fmt.Errorf("failed parsing pages_per_range %q: %w", kv[1], err)
+			}
+			params.PagesPerRange = i
+		}
+	}
+	return params, nil
+}
+
 const (
 	// Query to list runtime parameters.
-	paramsQuery = `SELECT setting FROM pg_settings WHERE name IN ('lc_collate', 'lc_ctype', 'server_version_num') ORDER BY name`
+	paramsQuery = `SELECT setting FROM pg_settings WHERE name IN ('lc_collate', 'lc_ctype', 'server_version_num', 'crdb_version') ORDER BY name DESC`
 
 	// Query to list database schemas.
-	schemasQuery = "SELECT schema_name FROM information_schema.schemata WHERE schema_name NOT IN ('information_schema', 'pg_catalog', 'pg_toast') ORDER BY schema_name"
+	schemasQuery = "SELECT schema_name FROM information_schema.schemata WHERE schema_name NOT IN ('information_schema', 'pg_catalog', 'pg_toast', 'crdb_internal', 'pg_extension') AND schema_name NOT LIKE 'pg_%temp_%' ORDER BY schema_name"
 
 	// Query to list specific database schemas.
 	schemasQueryArgs = "SELECT schema_name FROM information_schema.schemata WHERE schema_name %s ORDER BY schema_name"
 
-	// Query to list schema tables.
-	tablesQuery = "SELECT table_name FROM information_schema.tables WHERE table_type = 'BASE TABLE' AND table_schema = $1 ORDER BY table_name"
-
-	// Query to list specific schema tables.
-	tablesQueryArgs = "SELECT table_name FROM information_schema.tables WHERE table_type = 'BASE TABLE' AND table_schema = $1 AND table_name %s ORDER BY table_name"
-
 	// Query to list table information.
-	tableQuery = `
+	tablesQuery = `
 SELECT
 	t1.table_schema,
-	pg_catalog.obj_description(t2.oid, 'pg_class') AS COMMENT
-FROM
-	information_schema.tables AS t1
-	INNER JOIN pg_catalog.pg_class AS t2
-	ON t1.table_name = t2.relname
-WHERE
-	t1.table_type = 'BASE TABLE'
-	AND t1.table_name = $1
-	AND t1.table_schema = (CURRENT_SCHEMA())
-`
-	tableSchemaQuery = `
-SELECT
-	t1.TABLE_SCHEMA,
-	pg_catalog.obj_description(t2.oid, 'pg_class') AS COMMENT
+	t1.table_name,
+	pg_catalog.obj_description(t3.oid, 'pg_class') AS comment,
+	t4.partattrs AS partition_attrs,
+	t4.partstrat AS partition_strategy,
+	pg_get_expr(t4.partexprs, t4.partrelid) AS partition_exprs
 FROM
 	INFORMATION_SCHEMA.TABLES AS t1
-	JOIN pg_catalog.pg_class AS t2
-	ON t1.table_name = t2.relname
+	JOIN pg_catalog.pg_namespace AS t2 ON t2.nspname = t1.table_schema
+	JOIN pg_catalog.pg_class AS t3 ON t3.relnamespace = t2.oid AND t3.relname = t1.table_name
+	LEFT JOIN pg_catalog.pg_partitioned_table AS t4 ON t4.partrelid = t3.oid
 WHERE
-	t1.TABLE_TYPE = 'BASE TABLE'
-	AND t1.TABLE_NAME = $1
-	AND t1.TABLE_SCHEMA = $2
+	t1.table_type = 'BASE TABLE'
+	AND NOT COALESCE(t3.relispartition, false)
+	AND t1.table_schema IN (%s)
+ORDER BY
+	t1.table_schema, t1.table_name
+`
+	tablesQueryArgs = `
+SELECT
+	t1.table_schema,
+	t1.table_name,
+	pg_catalog.obj_description(t3.oid, 'pg_class') AS comment,
+	t4.partattrs AS partition_attrs,
+	t4.partstrat AS partition_strategy,
+	pg_get_expr(t4.partexprs, t4.partrelid) AS partition_exprs
+FROM
+	INFORMATION_SCHEMA.TABLES AS t1
+	JOIN pg_catalog.pg_namespace AS t2 ON t2.nspname = t1.table_schema
+	JOIN pg_catalog.pg_class AS t3 ON t3.relnamespace = t2.oid AND t3.relname = t1.table_name
+	LEFT JOIN pg_catalog.pg_partitioned_table AS t4 ON t4.partrelid = t3.oid
+WHERE
+	t1.table_type = 'BASE TABLE'
+	AND NOT COALESCE(t3.relispartition, false)
+	AND t1.table_schema IN (%s)
+	AND t1.table_name IN (%s)
+ORDER BY
+	t1.table_schema, t1.table_name
 `
 	// Query to list table columns.
 	columnsQuery = `
 SELECT
+	t1.table_name,
 	t1.column_name,
 	t1.data_type,
 	t1.is_nullable,
 	t1.column_default,
 	t1.character_maximum_length,
 	t1.numeric_precision,
+	t1.datetime_precision,
 	t1.numeric_scale,
 	t1.character_set_name,
 	t1.collation_name,
@@ -763,20 +895,26 @@ SELECT
 	t1.identity_start,
 	t1.identity_increment,
 	t1.identity_generation,
-	col_description(to_regclass("table_schema" || '.' || "table_name")::oid, "ordinal_position") AS comment,
-	t2.typtype,
-	t2.oid
+	t1.generation_expression,
+	col_description(t3.oid, "ordinal_position") AS comment,
+	t4.typtype,
+	t4.oid
 FROM
 	"information_schema"."columns" AS t1
-	LEFT JOIN pg_catalog.pg_type AS t2
-	ON t1.udt_name = t2.typname
+	JOIN pg_catalog.pg_namespace AS t2 ON t2.nspname = t1.table_schema
+	JOIN pg_catalog.pg_class AS t3 ON t3.relnamespace = t2.oid AND t3.relname = t1.table_name
+	LEFT JOIN pg_catalog.pg_type AS t4
+	ON t1.udt_name = t4.typname
 WHERE
-	TABLE_SCHEMA = $1 AND TABLE_NAME = $2
+	table_schema = $1 AND table_name IN (%s)
+ORDER BY
+	t1.table_name, t1.ordinal_position
 `
 
 	// Query to list table indexes.
 	indexesQuery = `
 SELECT
+	t.relname AS table_name,
 	i.relname AS index_name,
 	am.amname AS index_type,
 	a.attname AS column_name,
@@ -784,27 +922,32 @@ SELECT
 	idx.indisunique AS unique,
 	c.contype AS constraint_type,
 	pg_get_expr(idx.indpred, idx.indrelid) AS predicate,
-	pg_get_expr(idx.indexprs, idx.indrelid) AS expression,
-	pg_index_column_has_property(idx.indexrelid, a.attnum, 'asc') AS asc,
-	pg_index_column_has_property(idx.indexrelid, a.attnum, 'desc') AS desc,
-	pg_index_column_has_property(idx.indexrelid, a.attnum, 'nulls_first') AS nulls_first,
-	pg_index_column_has_property(idx.indexrelid, a.attnum, 'nulls_last') AS nulls_last,
-	obj_description(to_regclass($1 || i.relname)::oid) AS comment
+	pg_get_indexdef(idx.indexrelid, idx.ord, false) AS expression,
+	pg_index_column_has_property(idx.indexrelid, idx.ord, 'desc') AS desc,
+	pg_index_column_has_property(idx.indexrelid, idx.ord, 'nulls_first') AS nulls_first,
+	pg_index_column_has_property(idx.indexrelid, idx.ord, 'nulls_last') AS nulls_last,
+	obj_description(to_regclass($1 || i.relname)::oid) AS comment,
+	i.reloptions AS options
 FROM
-	pg_index idx
-	JOIN pg_class i
-	ON i.oid = idx.indexrelid
-	LEFT JOIN pg_constraint c
-	ON idx.indexrelid = c.conindid
-	LEFT JOIN pg_attribute a
-	ON a.attrelid = idx.indexrelid
-	JOIN pg_am am
-	ON am.oid = i.relam
+	(
+		select
+			*,
+			generate_series(1,array_length(i.indkey,1)) as ord,
+			unnest(i.indkey) AS key
+		from pg_index i
+	) idx
+	JOIN pg_class i ON i.oid = idx.indexrelid
+	JOIN pg_class t ON t.oid = idx.indrelid
+	JOIN pg_namespace n ON n.oid = t.relnamespace
+	LEFT JOIN pg_constraint c ON idx.indexrelid = c.conindid
+	LEFT JOIN pg_attribute a ON (a.attrelid, a.attnum) = (idx.indrelid, idx.key)
+	JOIN pg_am am ON am.oid = i.relam
 WHERE
-	idx.indrelid = to_regclass($1 || '.' || $2)::oid
+	n.nspname = $1
+	AND t.relname IN (%s)
 	AND COALESCE(c.contype, '') <> 'f'
 ORDER BY
-	index_name, a.attnum
+	table_name, index_name, idx.ord
 `
 	fksQuery = `
 SELECT
@@ -831,7 +974,7 @@ FROM
 WHERE
     t1.constraint_type = 'FOREIGN KEY'
     AND t1.table_schema = $1
-    AND t1.table_name = $2
+    AND t1.table_name IN (%s)
 ORDER BY
     t1.constraint_name,
     t2.ordinal_position
@@ -840,19 +983,25 @@ ORDER BY
 	// Query to list table check constraints.
 	checksQuery = `
 SELECT
+	rel.relname AS table_name,
 	t1.conname AS constraint_name,
-	pg_get_expr(t1.conbin, to_regclass($1 || '.' || $2)::oid) as expression,
+	pg_get_expr(t1.conbin, t1.conrelid) as expression,
 	t2.attname as column_name,
 	t1.conkey as column_indexes,
 	t1.connoinherit as no_inherit
 FROM
-	pg_catalog.pg_constraint t1
+	pg_constraint t1
 	JOIN pg_attribute t2
 	ON t2.attrelid = t1.conrelid
 	AND t2.attnum = ANY (t1.conkey)
+	JOIN pg_class rel
+	ON rel.oid = t1.conrelid
+	JOIN pg_namespace nsp
+	ON nsp.oid = t1.connamespace
 WHERE
 	t1.contype = 'c'
-	AND t1.conrelid = to_regclass($1 || '.' || $2)::oid
+	AND nsp.nspname = $1
+	AND rel.relname IN (%s)
 ORDER BY
 	t1.conname, array_position(t1.conkey, t2.attnum)
 `
