@@ -70,6 +70,41 @@ func (s *Schema) AddTables(tables ...*Table) *Schema {
 	return s
 }
 
+// NewRealm creates a new Realm.
+func NewRealm(schemas ...*Schema) *Realm {
+	r := &Realm{Schemas: schemas}
+	for _, s := range schemas {
+		s.Realm = r
+	}
+	return r
+}
+
+// SetCharset sets or appends the Charset attribute
+// to the realm with the given value.
+func (r *Realm) SetCharset(v string) *Realm {
+	replaceOrAppend(&r.Attrs, &Charset{V: v})
+	return r
+}
+
+// UnsetCharset unsets the Charset attribute.
+func (r *Realm) UnsetCharset() *Realm {
+	del(&r.Attrs, &Charset{})
+	return r
+}
+
+// SetCollation sets or appends the Collation attribute
+// to the realm with the given value.
+func (r *Realm) SetCollation(v string) *Realm {
+	replaceOrAppend(&r.Attrs, &Collation{V: v})
+	return r
+}
+
+// UnsetCollation the Collation attribute.
+func (r *Realm) UnsetCollation() *Realm {
+	del(&r.Attrs, &Collation{})
+	return r
+}
+
 // NewTable creates a new Table.
 func NewTable(name string) *Table {
 	return &Table{Name: name}
@@ -315,6 +350,13 @@ func DecimalScale(scale int) DecimalOption {
 	}
 }
 
+// DecimalUnsigned configures the unsigned of the float type.
+func DecimalUnsigned(unsigned bool) DecimalOption {
+	return func(b *DecimalType) {
+		b.Unsigned = unsigned
+	}
+}
+
 // NewDecimalColumn creates a new DecimalType column.
 func NewDecimalColumn(name, typ string, opts ...DecimalOption) *Column {
 	t := &DecimalType{T: typ}
@@ -340,6 +382,13 @@ func FloatPrecision(precision int) FloatOption {
 	}
 }
 
+// FloatUnsigned configures the unsigned of the float type.
+func FloatUnsigned(unsigned bool) FloatOption {
+	return func(b *FloatType) {
+		b.Unsigned = unsigned
+	}
+}
+
 // NewFloatColumn creates a new FloatType column.
 func NewFloatColumn(name, typ string, opts ...FloatOption) *Column {
 	t := &FloatType{T: typ}
@@ -355,10 +404,23 @@ func NewNullFloatColumn(name, typ string, opts ...FloatOption) *Column {
 		SetNull(true)
 }
 
+// TimeOption allows configuring TimeType using functional options.
+type TimeOption func(*TimeType)
+
+// TimePrecision configures the precision of the time type.
+func TimePrecision(precision int) TimeOption {
+	return func(b *TimeType) {
+		b.Precision = &precision
+	}
+}
+
 // NewTimeColumn creates a new TimeType column.
-func NewTimeColumn(name, typ string) *Column {
-	return NewColumn(name).
-		SetType(&TimeType{T: typ})
+func NewTimeColumn(name, typ string, opts ...TimeOption) *Column {
+	t := &TimeType{T: typ}
+	for _, opt := range opts {
+		opt(t)
+	}
+	return NewColumn(name).SetType(t)
 }
 
 // NewNullTimeColumn creates a new nullable TimeType column.
@@ -448,6 +510,12 @@ func (c *Column) SetComment(v string) *Column {
 	return c
 }
 
+// SetGeneratedExpr sets or appends the GeneratedExpr attribute.
+func (c *Column) SetGeneratedExpr(x *GeneratedExpr) *Column {
+	replaceOrAppend(&c.Attrs, x)
+	return c
+}
+
 // AddAttrs adds additional attributes to the column.
 func (c *Column) AddAttrs(attrs ...Attr) *Column {
 	c.Attrs = append(c.Attrs, attrs...)
@@ -558,14 +626,25 @@ func (i *Index) AddParts(parts ...*IndexPart) *Index {
 		if p.C != nil && !p.C.hasIndex(i) {
 			p.C.Indexes = append(p.C.Indexes, i)
 		}
+		p.SeqNo = len(i.Parts)
 		i.Parts = append(i.Parts, p)
 	}
 	return i
 }
 
 // NewIndexPart creates a new index part.
-func NewIndexPart(seqno int) *IndexPart {
-	return &IndexPart{SeqNo: seqno}
+func NewIndexPart() *IndexPart { return &IndexPart{} }
+
+// NewColumnPart creates a new index part with the given column.
+func NewColumnPart(c *Column) *IndexPart { return &IndexPart{C: c} }
+
+// NewExprPart creates a new index part with the given expression.
+func NewExprPart(x Expr) *IndexPart { return &IndexPart{X: x} }
+
+// SetDesc configures the "DESC" attribute of the key part.
+func (p *IndexPart) SetDesc(b bool) *IndexPart {
+	p.Desc = b
+	return p
 }
 
 // AddAttrs adds and additional attributes to the index-part.
@@ -655,6 +734,12 @@ func replaceOrAppend(attrs *[]Attr, v Attr) {
 	*attrs = append(*attrs, v)
 }
 
+// ReplaceOrAppend searches an attribute of the same type as v in
+// the list and replaces it. Otherwise, v is appended to the list.
+func ReplaceOrAppend(attrs *[]Attr, v Attr) {
+	replaceOrAppend(attrs, v)
+}
+
 // del searches an attribute of the same type as v in
 // the list and delete it.
 func del(attrs *[]Attr, v Attr) {
