@@ -17,7 +17,7 @@ import (
 	"ariga.io/atlas/sql/schema"
 )
 
-// A diff provides a PostgreSQL implementation for schema.Inspector.
+// A diff provides an SQLite implementation for schema.Inspector.
 type inspect struct{ conn }
 
 var _ schema.Inspector = (*inspect)(nil)
@@ -32,6 +32,9 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 		return nil, fmt.Errorf("sqlite: multiple database files are not supported by the driver. got: %d", len(schemas))
 	}
 	realm := &schema.Realm{Schemas: schemas}
+	if !sqlx.ModeInspectRealm(opts).Is(schema.InspectTables) {
+		return realm, nil
+	}
 	for _, s := range schemas {
 		tables, err := i.tables(ctx, nil)
 		if err != nil {
@@ -68,11 +71,15 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 			Err: fmt.Errorf("sqlite: schema %q was not found", name),
 		}
 	}
+	s := schemas[0]
+	s.Realm = schema.NewRealm(schemas...)
+	if !sqlx.ModeInspectSchema(opts).Is(schema.InspectTables) {
+		return s, nil
+	}
 	tables, err := i.tables(ctx, opts)
 	if err != nil {
 		return nil, err
 	}
-	s := schemas[0]
 	for _, t := range tables {
 		t.Schema = s
 		t, err := i.inspectTable(ctx, t)
@@ -82,29 +89,9 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 		s.Tables = append(s.Tables, t)
 	}
 	sqlx.LinkSchemaTables(schemas)
-	s.Realm = &schema.Realm{Schemas: schemas}
 	return s, nil
 }
 
-// InspectTable returns the schema description of the given table.
-func (i *inspect) InspectTable(ctx context.Context, name string, opts *schema.InspectTableOptions) (*schema.Table, error) {
-	if opts != nil && opts.Schema != mainFile {
-		return nil, fmt.Errorf("sqlite: querying attached database is not supported. got: %q", opts.Schema)
-	}
-	s, err := i.InspectSchema(ctx, mainFile, &schema.InspectOptions{
-		Tables: []string{name},
-	})
-	if err != nil {
-		return nil, err
-	}
-	if len(s.Tables) == 0 {
-		return nil, &schema.NotExistError{
-			Err: fmt.Errorf("sqlite: table %q was not found", name),
-		}
-	}
-	return s.Tables[0], nil
-}
-
 func (i *inspect) inspectTable(ctx context.Context, t *schema.Table) (*schema.Table, error) {
 	if err := i.columns(ctx, t); err != nil {
 		return nil, err
@@ -140,10 +127,11 @@ func (i *inspect) columns(ctx context.Context, t *schema.Table) error {
 func (i *inspect) addColumn(t *schema.Table, rows *sql.Rows) error {
 	var (
 		nullable, primary   bool
+		hidden              sql.NullInt64
 		name, typ, defaults sql.NullString
 		err                 error
 	)
-	if err = rows.Scan(&name, &typ, &nullable, &defaults, &primary); err != nil {
+	if err = rows.Scan(&name, &typ, &nullable, &defaults, &primary, &hidden); err != nil {
 		return err
 	}
 	c := &schema.Column{
@@ -157,10 +145,16 @@ func (i *inspect) addColumn(t *schema.Table, rows *sql.Rows) error {
 	if err != nil {
 		return err
 	}
-	if sqlx.ValidString(defaults) {
+	if defaults.Valid {
 		c.Default = defaultExpr(defaults.String)
 	}
-	// TODO(a8m): extract collation from 'CREATE TABLE' statement.
+	// The hidden flag is set to 2 for VIRTUAL columns, and to
+	// 3 for STORED columns. See: sqlite/pragma.c#sqlite3Pragma.
+	if hidden.Int64 >= 2 {
+		if err := setGenExpr(t, c, hidden.Int64); err != nil {
+			return err
+		}
+	}
 	t.Columns = append(t.Columns, c)
 	if primary {
 		if t.PrimaryKey == nil {
@@ -189,7 +183,7 @@ func (i *inspect) indexes(ctx context.Context, t *schema.Table) error {
 		return fmt.Errorf("sqlite: scan %q indexes: %w", t.Name, err)
 	}
 	for _, idx := range t.Indexes {
-		if err := i.indexColumns(ctx, t, idx); err != nil {
+		if err := i.indexInfo(ctx, t, idx); err != nil {
 			return err
 		}
 	}
@@ -233,33 +227,59 @@ func (i *inspect) addIndexes(t *schema.Table, rows *sql.Rows) error {
 	return nil
 }
 
-func (i *inspect) indexColumns(ctx context.Context, t *schema.Table, idx *schema.Index) error {
-	rows, err := i.QueryContext(ctx, fmt.Sprintf(indexColumnsQuery, idx.Name))
+// A regexp to extract index parts.
+var reIdxParts = regexp.MustCompile("(?i)ON\\s+[\"`]*(?:\\w+)[\"`]*\\s*\\((.+)\\)")
+
+func (i *inspect) indexInfo(ctx context.Context, t *schema.Table, idx *schema.Index) error {
+	var (
+		hasExpr   bool
+		rows, err = i.QueryContext(ctx, fmt.Sprintf(indexColumnsQuery, idx.Name))
+	)
 	if err != nil {
 		return fmt.Errorf("sqlite: querying %q indexes: %w", t.Name, err)
 	}
 	defer rows.Close()
 	for rows.Next() {
-		var name sql.NullString
-		if err := rows.Scan(&name); err != nil {
+		var (
+			desc sql.NullBool
+			name sql.NullString
+		)
+		if err := rows.Scan(&name, &desc); err != nil {
 			return fmt.Errorf("sqlite: scanning index names: %w", err)
 		}
+		part := &schema.IndexPart{
+			SeqNo: len(idx.Parts) + 1,
+			Desc:  desc.Bool,
+		}
 		switch c, ok := t.Column(name.String); {
 		case ok:
-			idx.Parts = append(idx.Parts, &schema.IndexPart{
-				SeqNo: len(idx.Parts) + 1,
-				C:     c,
-			})
+			part.C = c
 		// NULL name indicates that the index-part is an expression and we
 		// should extract it from the `CREATE INDEX` statement (not supported atm).
 		case !sqlx.ValidString(name):
-			idx.Parts = append(idx.Parts, &schema.IndexPart{
-				SeqNo: len(idx.Parts) + 1,
-				X:     &schema.RawExpr{X: "<unsupported>"},
-			})
+			hasExpr = true
+			part.X = &schema.RawExpr{X: "<unsupported>"}
 		default:
 			return fmt.Errorf("sqlite: column %q was not found for index %q", name.String, idx.Name)
 		}
+		idx.Parts = append(idx.Parts, part)
+	}
+	if !hasExpr {
+		return nil
+	}
+	var c CreateStmt
+	if !sqlx.Has(idx.Attrs, &c) || !reIdxParts.MatchString(c.S) {
+		return nil
+	}
+	parts := strings.Split(reIdxParts.FindStringSubmatch(c.S)[1], ",")
+	// Unable to parse index parts correctly.
+	if len(parts) != len(idx.Parts) {
+		return nil
+	}
+	for i, p := range idx.Parts {
+		if p.X != nil {
+			p.X.(*schema.RawExpr).X = strings.TrimSpace(parts[i])
+		}
 	}
 	return nil
 }
@@ -510,7 +530,7 @@ var reAutoinc = regexp.MustCompile("(?i)(?:[(,]\\s*)[\"`]?(\\w+)[\"`]?\\s+INTEGE
 func autoinc(t *schema.Table) error {
 	var c CreateStmt
 	if !sqlx.Has(t.Attrs, &c) {
-		return fmt.Errorf("missing CREATE statment for table: %q", t.Name)
+		return fmt.Errorf("missing CREATE statement for table: %q", t.Name)
 	}
 	if t.PrimaryKey == nil || len(t.PrimaryKey.Parts) != 1 || t.PrimaryKey.Parts[0].C == nil {
 		return nil
@@ -533,6 +553,33 @@ func autoinc(t *schema.Table) error {
 	return nil
 }
 
+// setGenExpr extracts the generated expression from the CREATE statement
+// and appends it to the column.
+func setGenExpr(t *schema.Table, c *schema.Column, f int64) error {
+	var s CreateStmt
+	if !sqlx.Has(t.Attrs, &s) {
+		return fmt.Errorf("missing CREATE statement for table: %q", t.Name)
+	}
+	re, err := regexp.Compile(fmt.Sprintf("(?:[(,]\\s*)[\"`]*(%s)[\"`]*[^,]*(?i:GENERATED\\s+ALWAYS)*\\s*(?i:AS){1}\\s*\\(", c.Name))
+	if err != nil {
+		return err
+	}
+	idx := re.FindAllStringIndex(s.S, 1)
+	if len(idx) != 1 || len(idx[0]) != 2 {
+		return fmt.Errorf("sqlite: generation expression for column %q was not found in create statement", c.Name)
+	}
+	expr := scanExpr(s.S[idx[0][1]-1:])
+	if expr == "" {
+		return fmt.Errorf("sqlite: unexpected empty generation expression for column %q", c.Name)
+	}
+	typ := virtual
+	if f == 3 {
+		typ = stored
+	}
+	c.SetGeneratedExpr(&schema.GeneratedExpr{Expr: expr, Type: typ})
+	return nil
+}
+
 // The following regexes extract named FKs and CHECK constraints defined in table-constraints or inlined
 // as column-constraints. Note, we assume the SQL statements are valid as they are returned by SQLite.
 var (
@@ -545,7 +592,7 @@ var (
 func fillConstName(t *schema.Table) error {
 	var c CreateStmt
 	if !sqlx.Has(t.Attrs, &c) {
-		return fmt.Errorf("missing CREATE statment for table: %q", t.Name)
+		return fmt.Errorf("missing CREATE statement for table: %q", t.Name)
 	}
 	// Loop over table constraints.
 	for _, m := range reFKT.FindAllStringSubmatch(c.S, -1) {
@@ -610,7 +657,7 @@ func matchFK(fk *schema.ForeignKey, columns []string, refTable string, refColumn
 func fillChecks(t *schema.Table) error {
 	var c CreateStmt
 	if !sqlx.Has(t.Attrs, &c) {
-		return fmt.Errorf("missing CREATE statment for table: %q", t.Name)
+		return fmt.Errorf("missing CREATE statement for table: %q", t.Name)
 	}
 	for i := 0; i < len(c.S); {
 		idx := reCheck.FindStringSubmatchIndex(c.S[i:])
@@ -618,52 +665,54 @@ func fillChecks(t *schema.Table) error {
 		if len(idx) != 4 {
 			break
 		}
-		var (
-			r, l int
-			expr = c.S[idx[1]-1:]
-		)
-		for i := 0; i < len(expr); i++ {
-			switch expr[i] {
-			case '(':
-				r++
-			case ')':
-				l++
-			case '\'', '"':
-				// Skip unescaped strings.
-				if j := strings.IndexByte(expr[i+1:], expr[i]); j != -1 {
-					i += j + 1
-				}
-			}
-			// Balanced parens.
-			if r == l {
-				check := &schema.Check{Expr: expr[:i+1]}
-				// Matching group for constraint name.
-				if idx[2] != -1 && idx[3] != -1 {
-					check.Name = c.S[idx[2]:idx[3]]
-				}
-				t.Attrs = append(t.Attrs, check)
-				idx[1] = idx[1] + i
-				break
-			}
+		check := &schema.Check{Expr: scanExpr(c.S[idx[1]-1:])}
+		// Matching group for constraint name.
+		if idx[2] != -1 && idx[3] != -1 {
+			check.Name = c.S[idx[2]:idx[3]]
 		}
-		c.S = c.S[idx[1]:]
+		t.Attrs = append(t.Attrs, check)
+		c.S = c.S[idx[1]+len(check.Expr)-1:]
 	}
 	return nil
 }
 
+// scanExpr scans the expression string (wrapped with parens)
+// until its end in the given string. e.g. "(a+1), c int ...".
+func scanExpr(expr string) string {
+	var r, l int
+	for i := 0; i < len(expr); i++ {
+		switch expr[i] {
+		case '(':
+			r++
+		case ')':
+			l++
+		case '\'', '"':
+			// Skip unescaped strings.
+			if j := strings.IndexByte(expr[i+1:], expr[i]); j != -1 {
+				i += j + 1
+			}
+		}
+		// Balanced parens.
+		if r == l {
+			return expr[:i+1]
+		}
+	}
+	return ""
+}
+
 const (
 	// Name of main database file.
 	mainFile = "main"
 	// Query to list attached database files.
 	databasesQuery = "SELECT `name`, `file` FROM pragma_database_list()"
 	// Query to list database tables.
-	tablesQuery = "SELECT `name`, `sql` FROM sqlite_master WHERE `type`='table' AND `name` NOT LIKE 'sqlite_%'"
+	tablesQuery = "SELECT `name`, `sql` FROM sqlite_master WHERE `type` = 'table' AND `name` NOT LIKE 'sqlite_%'"
 	// Query to list table information.
-	columnsQuery = "SELECT `name`, `type`, (not `notnull`) AS `nullable`, `dflt_value`, (`pk` <> 0) AS `pk`  FROM pragma_table_info('%s') ORDER BY `pk`, `cid`"
+	columnsQuery = "SELECT `name`, `type`, (not `notnull`) AS `nullable`, `dflt_value`, (`pk` <> 0) AS `pk`, `hidden` FROM pragma_table_xinfo('%s') ORDER BY `pk`, `cid`"
 	// Query to list table indexes.
 	indexesQuery = "SELECT `il`.`name`, `il`.`unique`, `il`.`origin`, `il`.`partial`, `m`.`sql` FROM pragma_index_list('%s') AS il JOIN sqlite_master AS m ON il.name = m.name"
 	// Query to list index columns.
-	indexColumnsQuery = "SELECT name FROM pragma_index_info('%s') ORDER BY seqno"
+	indexColumnsQuery = "SELECT name, desc FROM pragma_index_xinfo('%s') WHERE key = 1 ORDER BY seqno"
 	// Query to list table foreign-keys.
 	fksQuery = "SELECT `id`, `from`, `to`, `table`, `on_update`, `on_delete` FROM pragma_foreign_key_list('%s') ORDER BY id, seq"
 )
