@@ -1,21 +1,24 @@
+// Copyright 2021-present The Atlas Authors. All rights reserved.
+// This source code is licensed under the Apache 2.0 license found
+// in the LICENSE file in the root directory of this source tree.
+
 package sqlspec
 
 import (
 	"ariga.io/atlas/schema/schemaspec"
-	"ariga.io/atlas/sql/schema"
 )
 
 type (
-
 	// Schema holds a specification for a Schema.
 	Schema struct {
-		Name string `spec:",name"`
+		Name string `spec:"name,name"`
 		schemaspec.DefaultExtension
 	}
 
 	// Table holds a specification for an SQL table.
 	Table struct {
 		Name        string          `spec:",name"`
+		Qualifier   string          `spec:",qualifier"`
 		Schema      *schemaspec.Ref `spec:"schema"`
 		Columns     []*Column       `spec:"column"`
 		PrimaryKey  *PrimaryKey     `spec:"primary_key"`
@@ -43,11 +46,20 @@ type (
 	// Index holds a specification for the index key of a table.
 	Index struct {
 		Name    string            `spec:",name"`
-		Unique  bool              `spec:"unique"`
+		Unique  bool              `spec:"unique,omitempty"`
+		Parts   []*IndexPart      `spec:"on"`
 		Columns []*schemaspec.Ref `spec:"columns"`
 		schemaspec.DefaultExtension
 	}
 
+	// IndexPart holds a specification for the index key part.
+	IndexPart struct {
+		Desc   bool            `spec:"desc,omitempty"`
+		Column *schemaspec.Ref `spec:"column"`
+		Expr   string          `spec:"expr,omitempty"`
+		schemaspec.DefaultExtension
+	}
+
 	// Check holds a specification for a check constraint on a table.
 	Check struct {
 		Name string `spec:",name"`
@@ -57,11 +69,11 @@ type (
 
 	// ForeignKey holds a specification for the Foreign key of a table.
 	ForeignKey struct {
-		Symbol     string                 `spec:",name"`
-		Columns    []*schemaspec.Ref      `spec:"columns"`
-		RefColumns []*schemaspec.Ref      `spec:"ref_columns"`
-		OnUpdate   schema.ReferenceOption `spec:"on_update"`
-		OnDelete   schema.ReferenceOption `spec:"on_delete"`
+		Symbol     string            `spec:",name"`
+		Columns    []*schemaspec.Ref `spec:"columns"`
+		RefColumns []*schemaspec.Ref `spec:"ref_columns"`
+		OnUpdate   *schemaspec.Ref   `spec:"on_update"`
+		OnDelete   *schemaspec.Ref   `spec:"on_delete"`
 		schemaspec.DefaultExtension
 	}
 
@@ -69,26 +81,6 @@ type (
 	Type string
 )
 
-// Type<X> are Types that represent database agnostic column types to be used
-// in Atlas DDL documents.
-const (
-	TypeInt     Type = "int"
-	TypeInt8    Type = "int8"
-	TypeInt16   Type = "int16"
-	TypeInt64   Type = "int64"
-	TypeUint    Type = "uint"
-	TypeUint8   Type = "uint8"
-	TypeUint16  Type = "uint16"
-	TypeUint64  Type = "uint64"
-	TypeString  Type = "string"
-	TypeBinary  Type = "binary"
-	TypeEnum    Type = "enum"
-	TypeBoolean Type = "boolean"
-	TypeDecimal Type = "decimal"
-	TypeFloat   Type = "float"
-	TypeTime    Type = "time"
-)
-
 func init() {
 	schemaspec.Register("table", &Table{})
 	schemaspec.Register("schema", &Schema{})
